@@ -1,11 +1,11 @@
 /***************************************************************************
- * CVSID: $Id: hald.h,v 1.8 2005/02/28 04:53:15 david Exp $
+ * CVSID: $Id: hald.h,v 1.11 2005/11/02 15:38:13 david Exp $
  *
  * device_store.h : device store interface
  *
  * Copyright (C) 2003 David Zeuthen, <david@fubar.dk>
  *
- * Licensed under the Academic Free License version 2.0
+ * Licensed under the Academic Free License version 2.1
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -19,7 +19,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  *
  **************************************************************************/
 
@@ -45,6 +45,7 @@ void property_atomic_update_begin ();
 void property_atomic_update_end ();
 
 extern dbus_bool_t hald_is_verbose;
+extern dbus_bool_t hald_use_syslog;
 extern dbus_bool_t hald_is_initialising;
 extern dbus_bool_t hald_is_shutting_down;
 
