@@ -1,11 +1,12 @@
 /***************************************************************************
- * CVSID: $Id: blockdev.c,v 1.18 2005/07/11 17:32:05 david Exp $
+ * CVSID: $Id: blockdev.c,v 1.38 2006/02/24 04:29:06 david Exp $
  *
  * blockdev.c : Handling of block devices
  *
  * Copyright (C) 2005 David Zeuthen, <david@fubar.dk>
+ * Copyright (C) 2005,2006 Kay Sievers, <kay.sievers@vrfy.org>
  *
- * Licensed under the Academic Free License version 2.0
+ * Licensed under the Academic Free License version 2.1
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -19,7 +20,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  *
  **************************************************************************/
 
@@ -33,7 +34,7 @@
 #include <string.h>
 #include <mntent.h>
 #include <errno.h>
-#include <sys/types.h>
+#include <stdint.h>
 #include <sys/stat.h>
 #include <sys/socket.h>
 #include <sys/un.h>
@@ -45,7 +46,7 @@
 
 #include <limits.h>
 #include <errno.h>
-#include <sys/types.h>
+#include <stdint.h>
 #include <sys/stat.h>
 #include <sys/ioctl.h>
 #include <fcntl.h>
@@ -60,6 +61,7 @@
 #include "../device_info.h"
 #include "../hald_dbus.h"
 #include "../util.h"
+#include "../hald_runner.h"
 
 #include "osspec_linux.h"
 
@@ -88,7 +90,13 @@ blockdev_compute_udi (HalDevice *d)
 					      "/org/freedesktop/Hal/devices/volume_uuid_%s", uuid);
 		} else if (label != NULL && strlen (label) > 0) {
 			hal_util_compute_udi (hald_get_gdl (), udi, sizeof (udi),
-					      "/org/freedesktop/Hal/devices/volume_label_%s", uuid);
+					      "/org/freedesktop/Hal/devices/volume_label_%s", label);
+		} else if (hal_device_property_get_bool(d, "volume.is_disc") &&
+			   hal_device_property_get_bool(d, "volume.disc.is_blank")) {
+			/* this should be a empty CD/DVD */
+			hal_util_compute_udi (hald_get_gdl (), udi, sizeof (udi),
+                                             "/org/freedesktop/Hal/devices/volume_empty_%s",
+					      hal_device_property_get_string (d, "volume.disc.type"));
 		} else {
 			/* fallback to partition number, size */
 			hal_util_compute_udi (hald_get_gdl (), udi, sizeof (udi),
@@ -154,85 +162,125 @@ blockdev_callouts_remove_done (HalDevice *d, gpointer userdata1, gpointer userda
 	hotplug_event_end (end_token);
 }
 
-static void 
-update_mount_point (HalDevice *d)
+static void
+cleanup_mountpoint_cb (HalDevice *d, guint32 exit_type, 
+		       gint return_code, gchar **error,
+		       gpointer data1, gpointer data2)
+{
+	char *mount_point = (char *) data1;
+	HAL_INFO (("In cleanup_mountpoint_cb for '%s'", mount_point));
+	g_free (mount_point);
+}
+
+
+void
+blockdev_refresh_mount_state (HalDevice *d)
 {
 	FILE *f;
 	struct mntent mnt;
 	struct mntent *mnte;
-	const char *device_file;
-	char buf[512];
-	unsigned int major, minor;
-	
-	if ((device_file = hal_device_property_get_string (d, "block.device")) == NULL)
-		goto out;
+	char buf[1024];
+	unsigned int major;
+	unsigned int minor;
+	dev_t devt = makedev(0, 0);
+	GSList *volumes = NULL;
+	GSList *volume;
 
-	major = hal_device_property_get_int (d, "block.major");
-	minor = hal_device_property_get_int (d, "block.minor");
-
-	HAL_INFO (("Update mount point for %s (device_file %s)", d->udi, device_file));
-
-	snprintf (buf, sizeof (buf), "%s/mounts", get_hal_proc_path ());
+	/* open /proc/mounts */
+	g_snprintf (buf, sizeof (buf), "%s/mounts", get_hal_proc_path ());
 	if ((f = setmntent (buf, "r")) == NULL) {
 		HAL_ERROR (("Could not open /proc/mounts"));
-		goto out;
+		return;
 	}
 
+	if (d)
+		volumes = g_slist_append (NULL, d);
+	else
+		volumes = hal_device_store_match_multiple_key_value_string (hald_get_gdl (), "info.category", "volume");
+
+	if (!volumes)
+		goto exit;
+
+	/* loop over /proc/mounts */
 	while ((mnte = getmntent_r (f, &mnt, buf, sizeof(buf))) != NULL) {
 		struct stat statbuf;
 
-		if (stat (mnt.mnt_fsname, &statbuf) != 0)
+		/* check the underlying device of the mount point */
+		if (stat (mnt.mnt_dir, &statbuf) != 0)
+			continue;
+		if (major(statbuf.st_dev) == 0)
 			continue;
 
-		if ((major (statbuf.st_rdev) == major) && (minor (statbuf.st_rdev) == minor)) {
-			device_property_atomic_update_begin ();
-			hal_device_property_set_bool (d, "volume.is_mounted", TRUE);
-			hal_device_property_set_string (d, "volume.mount_point", mnt.mnt_dir);
-			device_property_atomic_update_end ();
-			HAL_INFO (("Setting mount point %s for %s", mnt.mnt_dir, device_file));
-			goto found;
+		HAL_INFO (("* found mounts dev %s (%i:%i)", mnt.mnt_fsname, major(statbuf.st_dev), minor(statbuf.st_dev)));
+		/* match against all hal volumes */
+		for (volume = volumes; volume != NULL; volume = g_slist_next (volume)) {
+			HalDevice *dev;
+
+			dev = HAL_DEVICE (volume->data);
+			major = hal_device_property_get_int (dev, "block.major");
+			if (major == 0)
+				continue;
+			minor = hal_device_property_get_int (dev, "block.minor");
+			devt = makedev(major, minor);
+			HAL_INFO (("  match %s (%i:%i)", hal_device_get_udi (dev), major, minor));
+
+			if (statbuf.st_dev == devt) {
+				/* found entry for this device in /proc/mounts */
+				device_property_atomic_update_begin ();
+				hal_device_property_set_bool (dev, "volume.is_mounted", TRUE);
+				hal_device_property_set_string (dev, "volume.mount_point", mnt.mnt_dir);
+				device_property_atomic_update_end ();
+				HAL_INFO (("  set %s to be mounted at %s", hal_device_get_udi (dev), mnt.mnt_dir));
+				volumes = g_slist_delete_link (volumes, volume);
+				break;
+			}
 		}
 	}
 
-	device_property_atomic_update_begin ();
-	hal_device_property_set_bool (d, "volume.is_mounted", FALSE);
-	hal_device_property_set_string (d, "volume.mount_point", "");
-	device_property_atomic_update_end ();
+	/* all remaining volumes are not mounted */
+	for (volume = volumes; volume != NULL; volume = g_slist_next (volume)) {
+		HalDevice *dev;
+		char *mount_point;
+		char *mount_point_hal_file;
 
-	HAL_INFO (("Clearing mount point for %s", device_file));
+		dev = HAL_DEVICE (volume->data);
+		mount_point = g_strdup (hal_device_property_get_string (dev, "volume.mount_point"));
+		device_property_atomic_update_begin ();
+		hal_device_property_set_bool (dev, "volume.is_mounted", FALSE);
+		hal_device_property_set_string (dev, "volume.mount_point", "");
+		device_property_atomic_update_end ();
+		HAL_INFO (("set %s to unmounted", hal_device_get_udi (dev)));
 
-found:		
-	endmntent (f);
-out:
-	;
-}
+		mount_point_hal_file = g_strdup_printf ("%s/.created-by-hal", mount_point);
+		if (g_file_test (mount_point_hal_file, G_FILE_TEST_EXISTS)) {
+			char *cleanup_stdin;
+			char *extra_env[2];
 
-void 
-blockdev_mount_status_changed (const gchar *sysfs_path, gboolean is_mounted)
-{
-	HalDevice *d;
-	HAL_INFO (("mount_status_changed for '%s', is_mounted=%d", sysfs_path, is_mounted));
+			HAL_INFO (("Cleaning up directory '%s' since it was created by hal Mount()", mount_point));
 
-	if ((d = hal_device_store_match_key_value_string (hald_get_gdl (), "linux.sysfs_path", sysfs_path)) == NULL)
-		goto error;
+			extra_env[0] = g_strdup_printf ("HALD_CLEANUP=%s", mount_point);
+			extra_env[1] = NULL;
+			cleanup_stdin = "\n";
 
-	if (!hal_device_has_capability (d, "volume")) {
-		/* may have a fakevolume */
-		d = hal_device_store_match_key_value_string (hald_get_gdl (),
-							     "info.parent",
-							     d->udi);
-		if (d == NULL || !hal_device_has_capability (d, "volume"))
-			goto error;
+			hald_runner_run_method (dev, 
+						"hal-system-storage-cleanup-mountpoint", 
+						extra_env, 
+						cleanup_stdin, TRUE,
+						0,
+						cleanup_mountpoint_cb,
+						g_strdup (mount_point), NULL);
+
+			hal_device_property_remove (dev, "info.hal_mount.created_mount_point");
+			hal_device_property_remove (dev, "info.hal_mount.mounted_by_uid");
+
+		}
+
+		g_free (mount_point_hal_file);
+		g_free (mount_point);
 	}
-
-	HAL_INFO (("Applies to %s", d->udi));
-
-	update_mount_point (d);
-	return;
-
-error:
-	HAL_INFO (("Couldn't find hal volume for %s", sysfs_path));
-	;
+	g_slist_free (volumes);
+exit:
+	endmntent (f);
 }
 
 static void
@@ -264,15 +312,16 @@ generate_fakevolume_hotplug_event_add_for_storage_device (HalDevice *d)
 }
 
 static void 
-add_blockdev_probing_helper_done (HalDevice *d, gboolean timed_out, gint return_code, 
-				  gpointer data1, gpointer data2, HalHelperData *helper_data)
+add_blockdev_probing_helper_done (HalDevice *d, guint32 exit_type, 
+                                  gint return_code, char **error,
+                                  gpointer data1, gpointer data2) 
 {
 	void *end_token = (void *) data1;
 	gboolean is_volume;
 
 	/* helper_data may be null if probing is skipped */
 
-	HAL_INFO (("entering; timed_out=%d, return_code=%d", timed_out, return_code));
+	HAL_INFO (("entering; exit_type=%d, return_code=%d", exit_type, return_code));
 
 	if (d == NULL) {
 		HAL_INFO (("Device object already removed"));
@@ -286,7 +335,8 @@ add_blockdev_probing_helper_done (HalDevice *d, gboolean timed_out, gint return_
 	 * 
 	 * (return code 2 means fs found on main block device (for non-volumes)) 
 	 */
-	if (timed_out || !(return_code == 0 || (!is_volume && return_code == 2))) {
+	if (exit_type != HALD_RUN_SUCCESS
+	    || !(return_code == 0 || (!is_volume && return_code == 2))) {
 		hal_device_store_remove (hald_get_tdl (), d);
 		g_object_unref (d);
 		hotplug_event_end (end_token);
@@ -305,13 +355,13 @@ add_blockdev_probing_helper_done (HalDevice *d, gboolean timed_out, gint return_
 		hal_device_copy_property (d, "info.udi", d, "block.storage_device");
 	} else {
 		/* check for mount point */
-		update_mount_point (d);
+		blockdev_refresh_mount_state (d);
 	}
 
 	/* Merge properties from .fdi files */
 	di_search_and_merge (d, DEVICE_INFO_TYPE_INFORMATION);
 	di_search_and_merge (d, DEVICE_INFO_TYPE_POLICY);
-	
+
 	/* TODO: Merge persistent properties */
 
 	/* Run callouts */
@@ -327,7 +377,7 @@ add_blockdev_probing_helper_done (HalDevice *d, gboolean timed_out, gint return_
 
 
 out:
-	;
+	return;
 }
 
 static void 
@@ -345,7 +395,7 @@ blockdev_callouts_preprobing_storage_done (HalDevice *d, gpointer userdata1, gpo
 		hal_device_property_set_string (d, "info.product", "Ignored Device");
 
 		HAL_INFO (("Preprobing merged info.ignore==TRUE"));
-		
+
 		/* Move from temporary to global device store */
 		hal_device_store_remove (hald_get_tdl (), d);
 		hal_device_store_add (hald_get_gdl (), d);
@@ -357,34 +407,31 @@ blockdev_callouts_preprobing_storage_done (HalDevice *d, gpointer userdata1, gpo
 	if (!hal_device_property_get_bool (d, "storage.media_check_enabled") &&
 	    hal_device_property_get_bool (d, "storage.no_partitions_hint")) {
 
-
 		/* special probe for PC floppy drives */
 		if (strcmp (hal_device_property_get_string (d, "storage.bus"), "platform") == 0 &&
 		    strcmp (hal_device_property_get_string (d, "storage.drive_type"), "floppy") == 0) {
 			HAL_INFO (("Probing PC floppy %s to see if it is present", 
 				   hal_device_property_get_string (d, "block.device")));
 
-			if (hal_util_helper_invoke ("hald-probe-pc-floppy", NULL, d, (gpointer) end_token, 
-						    NULL, add_blockdev_probing_helper_done, 
-						    HAL_HELPER_TIMEOUT) == NULL) {
-				hal_device_store_remove (hald_get_tdl (), d);
-				g_object_unref (d);
-				hotplug_event_end (end_token);
-			}
+			hald_runner_run(d, 
+			                    "hald-probe-pc-floppy", NULL,
+			                    HAL_HELPER_TIMEOUT,
+			                    add_blockdev_probing_helper_done,
+			                    (gpointer) end_token, NULL);
 			goto out;
-
 		} else {
+			char *synerror[1] = {NULL};
+
 			HAL_INFO (("Not probing storage device %s", 
 				   hal_device_property_get_string (d, "block.device")));
 
-			add_blockdev_probing_helper_done (d, FALSE, 0, (gpointer) end_token, NULL, NULL);
+			add_blockdev_probing_helper_done (d, FALSE, 0, synerror, (gpointer) end_token, NULL);
 			goto out;
 		}
 	}
 
 	/* run prober for 
 	 *
-	 *  - drive_id
 	 *  - cdrom drive properties
 	 *  - non-partitioned filesystem on main block device
 	 */
@@ -392,25 +439,23 @@ blockdev_callouts_preprobing_storage_done (HalDevice *d, gpointer userdata1, gpo
 	HAL_INFO (("Probing storage device %s", hal_device_property_get_string (d, "block.device")));
 
 	/* probe the device */
-	if (hal_util_helper_invoke ("hald-probe-storage", NULL, d, (gpointer) end_token, 
-				    NULL, add_blockdev_probing_helper_done, 
-				    HAL_HELPER_TIMEOUT) == NULL) {
-		hal_device_store_remove (hald_get_tdl (), d);
-		g_object_unref (d);
-		hotplug_event_end (end_token);
-	}
+	hald_runner_run(d,
+			"hald-probe-storage", NULL,
+			HAL_HELPER_TIMEOUT,
+			add_blockdev_probing_helper_done,
+			(gpointer) end_token, NULL);
 
 out:
-	;
+	return;
 }
 
-static void 
+static void
 blockdev_callouts_preprobing_volume_done (HalDevice *d, gpointer userdata1, gpointer userdata2)
 {
 	void *end_token = (void *) userdata1;
 
 	if (hal_device_property_get_bool (d, "info.ignore")) {
-		/* Leave the device here with info.ignore==TRUE so we won't pick up children 
+		/* Leave the device here with info.ignore==TRUE so we won't pick up children
 		 * Also remove category and all capabilities
 		 */
 		hal_device_property_remove (d, "info.category");
@@ -419,26 +464,23 @@ blockdev_callouts_preprobing_volume_done (HalDevice *d, gpointer userdata1, gpoi
 		hal_device_property_set_string (d, "info.product", "Ignored Device");
 
 		HAL_INFO (("Preprobing merged info.ignore==TRUE"));
-		
+
 		/* Move from temporary to global device store */
 		hal_device_store_remove (hald_get_tdl (), d);
 		hal_device_store_add (hald_get_gdl (), d);
-		
+
 		hotplug_event_end (end_token);
 		goto out;
 	}
 
 	/* probe the device */
-	if (hal_util_helper_invoke ("hald-probe-volume", NULL, d, (gpointer) end_token, 
-				    NULL, add_blockdev_probing_helper_done, 
-				    HAL_HELPER_TIMEOUT) == NULL) {
-		hal_device_store_remove (hald_get_tdl (), d);
-		g_object_unref (d);
-		hotplug_event_end (end_token);
-	}
-
+	hald_runner_run (d,
+			 "hald-probe-volume", NULL, 
+			 HAL_HELPER_TIMEOUT,
+			 add_blockdev_probing_helper_done,
+			 (gpointer) end_token, NULL);
 out:
-	;
+	return;
 }
 
 static const gchar *
@@ -478,9 +520,10 @@ blockdev_get_luks_uuid(const gchar *device_file)
 			struct stat statbuf;
 			while ((f = g_dir_read_name (dir)) != NULL) {
 				char luks_prefix[] = "luks_crypto_";
-				HAL_INFO (("looking at /dev/mapper/%s", f));
 				g_snprintf (devpath, sizeof (devpath), "/dev/mapper/%s", f);
 				if (stat (devpath, &statbuf) == 0) {
+					HAL_INFO (("looking at /dev/mapper/%s with %d:%d", 
+						   f, MAJOR(statbuf.st_rdev), MINOR(statbuf.st_rdev)));
 					if (S_ISBLK (statbuf.st_mode) && 
 					    MAJOR(statbuf.st_rdev) == major && 
 					    MINOR(statbuf.st_rdev) == minor &&
@@ -529,6 +572,7 @@ void
 hotplug_event_begin_add_blockdev (const gchar *sysfs_path, const gchar *device_file, gboolean is_partition,
 				  HalDevice *parent, void *end_token)
 {
+	HotplugEvent *hotplug_event = (HotplugEvent *) end_token;
 	gchar *major_minor;
 	HalDevice *d;
 	unsigned int major, minor;
@@ -564,36 +608,28 @@ hotplug_event_begin_add_blockdev (const gchar *sysfs_path, const gchar *device_f
 
 	d = hal_device_new ();
 
-	/* lip service for PC floppy drives */
-	if (parent == NULL && sscanf (hal_util_get_last_element (sysfs_path), "fd%d", &floppy_num) == 1) {
-		;
-	} else {
-		floppy_num = -1;
-
-		if (parent == NULL) {
-			const gchar *luks_uuid = blockdev_get_luks_uuid (device_file);
-			if (luks_uuid != NULL) {
-				is_partition = TRUE;
-				parent = blockdev_get_luks_parent (luks_uuid, d);
-			}
+	if (parent == NULL) {
+		const gchar *luks_uuid = blockdev_get_luks_uuid (device_file);
+		if (luks_uuid != NULL) {
+			is_partition = TRUE;
+			parent = blockdev_get_luks_parent (luks_uuid, d);
 		}
+	}
 
-		if (parent == NULL) {
-			HAL_INFO (("Ignoring hotplug event - no parent"));
-			goto error;
-		}
+	if (parent == NULL) {
+		HAL_INFO (("Ignoring hotplug event - no parent"));
+		goto error;
+	}
 
-		if (!is_fakevolume && hal_device_property_get_bool (parent, "storage.no_partitions_hint")) {
-			HAL_INFO (("Ignoring blockdev since not a fakevolume and parent has "
-				   "storage.no_partitions_hint==TRUE"));
-			goto error;
-		}
+	if (!is_fakevolume && hal_device_property_get_bool (parent, "storage.no_partitions_hint")) {
+		HAL_INFO (("Ignoring blockdev since not a fakevolume and parent has "
+			   "storage.no_partitions_hint==TRUE"));
+		goto error;
 	}
 
 	hal_device_property_set_string (d, "linux.sysfs_path", sysfs_path);
 	hal_device_property_set_string (d, "linux.sysfs_path_device", sysfs_path);
-	if (parent != NULL)
-		hal_device_property_set_string (d, "info.parent", parent->udi);
+	hal_device_property_set_string (d, "info.parent", parent->udi);
 	hal_device_property_set_int (d, "linux.hotplug_type", HOTPLUG_EVENT_SYSFS_BLOCK);
 
 	hal_device_property_set_string (d, "block.device", device_file);
@@ -607,7 +643,9 @@ hotplug_event_begin_add_blockdev (const gchar *sysfs_path, const gchar *device_f
 	hal_device_property_set_int (d, "block.minor", minor);
 	hal_device_property_set_bool (d, "block.is_volume", is_partition);
 
-	if (floppy_num >= 0) {
+	if (hal_device_has_property(parent, "info.bus") &&
+		(strcmp(hal_device_property_get_string(parent, "info.bus"), "platform") == 0) &&
+		(sscanf(hal_device_property_get_string(parent, "platform.id"), "floppy.%d", &floppy_num) == 1)) {
 		/* for now, just cheat here for floppy drives */
 
 		HAL_INFO (("doing floppy drive hack for floppy %d", floppy_num));
@@ -621,8 +659,7 @@ hotplug_event_begin_add_blockdev (const gchar *sysfs_path, const gchar *device_f
 		hal_device_property_set_string (d, "info.vendor", "");
 		hal_device_property_set_string (d, "info.product", "PC Floppy Drive");
 		hal_device_property_set_string (d, "storage.drive_type", "floppy");
-		hal_device_property_set_string (d, "storage.physical_device", "/org/freedesktop/Hal/devices/computer");
-		hal_device_property_set_string (d, "info.parent", "/org/freedesktop/Hal/devices/computer");
+		hal_device_property_set_string (d, "storage.physical_device", parent->udi);
 		hal_device_property_set_bool (d, "storage.removable", TRUE);
 		hal_device_property_set_bool (d, "storage.hotpluggable", FALSE);
 		hal_device_property_set_bool (d, "storage.requires_eject", FALSE);
@@ -660,7 +697,7 @@ hotplug_event_begin_add_blockdev (const gchar *sysfs_path, const gchar *device_f
 
 		scsidev = NULL;
 		physdev = NULL;
-		physdev_udi = NULL;		
+		physdev_udi = NULL;
 
 		is_removable = FALSE;
 		is_hotpluggable = FALSE;
@@ -672,10 +709,16 @@ hotplug_event_begin_add_blockdev (const gchar *sysfs_path, const gchar *device_f
 		hal_device_property_set_bool (d, "storage.no_partitions_hint", TRUE);
 		hal_device_property_set_bool (d, "storage.media_check_enabled", TRUE);
 		hal_device_property_set_bool (d, "storage.automount_enabled_hint", TRUE);
-		hal_device_property_set_string (d, "storage.model", "");
-		hal_device_property_set_string (d, "storage.vendor", "");
 		hal_device_property_set_string (d, "storage.drive_type", "disk");
 
+		/* persistent properties from udev (may be empty) */
+		hal_device_property_set_string (d, "storage.model", hotplug_event->sysfs.model);
+		hal_device_property_set_string (d, "storage.vendor", hotplug_event->sysfs.vendor);
+		if (hotplug_event->sysfs.serial[0] != '\0')
+			hal_device_property_set_string (d, "storage.serial", hotplug_event->sysfs.serial);
+		if (hotplug_event->sysfs.revision[0] != '\0')
+			hal_device_property_set_string (d, "storage.firmware_version", hotplug_event->sysfs.revision);
+
 		/* walk up the device chain to find the physical device, 
 		 * start with our parent. On the way, optionally pick up
 		 * the scsi if it exists */
@@ -724,6 +767,7 @@ hotplug_event_begin_add_blockdev (const gchar *sysfs_path, const gchar *device_f
 				} else if (strcmp (bus, "mmc") == 0) {
 					physdev = d_it;
 					physdev_udi = udi_it;
+					is_hotpluggable = TRUE;
 					hal_device_property_set_string (d, "storage.bus", "mmc");
 					break;
 				} else if (strcmp (bus, "ccw") == 0) {
@@ -733,7 +777,6 @@ hotplug_event_begin_add_blockdev (const gchar *sysfs_path, const gchar *device_f
 					hal_device_property_set_string
 						(d, "storage.bus", "ccw");
 				}
-									
 			}
 
 			/* Go to parent */
@@ -741,13 +784,17 @@ hotplug_event_begin_add_blockdev (const gchar *sysfs_path, const gchar *device_f
 		}
 
 		/* needs physical device */
-		if (physdev_udi == NULL)
+		if (physdev_udi == NULL) {
+			HAL_WARNING (("No physical device?"));
 			goto error;
+		}
 
 		hal_device_property_set_string (d, "storage.physical_device", physdev_udi);
 
-		if (!hal_util_get_int_from_file (sysfs_path, "removable", (gint *) &is_removable, 10))
+		if (!hal_util_get_int_from_file (sysfs_path, "removable", (gint *) &is_removable, 10)) {
+			HAL_WARNING (("Cannot get 'removable' file"));
 			goto error;
+		}
 
 		hal_device_property_set_bool (d, "storage.removable", is_removable);
 
@@ -780,6 +827,7 @@ hotplug_event_begin_add_blockdev (const gchar *sysfs_path, const gchar *device_f
 				    strcmp (media, "floppy") == 0) {
 					hal_device_property_set_string (d, "storage.drive_type", media);
 				} else {
+					HAL_WARNING (("Cannot determine IDE drive type from file %s/media", buf));
 					goto error;
 				}
 
@@ -795,29 +843,16 @@ hotplug_event_begin_add_blockdev (const gchar *sysfs_path, const gchar *device_f
 			}
 
 		} else if (strcmp (parent_bus, "scsi") == 0) {
-			gint type;
-
-			if (hal_util_set_string_from_file (d, "storage.vendor", sysfs_path, "device/vendor"))
-				hal_device_copy_property (d, "storage.vendor", d, "info.vendor");
-			if (hal_util_set_string_from_file (d, "storage.model", sysfs_path, "device/model"))
-				hal_device_copy_property (d, "storage.model", d, "info.product");
-
-			if (!hal_util_get_int_from_file (sysfs_path, "device/type", &type, 0))
-				goto error;
-
-			/* These magic values are documented in the kernel source */
-			switch (type) {
-			case 0:	/* Disk */
-				hal_device_property_set_string (d, "storage.drive_type", "disk");
-				break;
-
-			case 5:	/* CD-ROM */
-				hal_device_property_set_string (d, "storage.drive_type", "cdrom");
-				break;
-
-			default:
+			if (strcmp (hal_device_property_get_string (parent, "scsi.type"), "unknown") == 0) {
+				HAL_WARNING (("scsi.type is unknown"));
 				goto error;
 			}
+			hal_device_copy_property (parent, "scsi.type", d, "storage.drive_type");
+			hal_device_copy_property (parent, "scsi.vendor", d, "storage.vendor");
+			hal_device_copy_property (parent, "scsi.model", d, "storage.model");
+
+			hal_device_copy_property (d, "storage.vendor", d, "info.vendor");
+			hal_device_copy_property (d, "storage.model", d, "info.product");
 
 			/* Check for USB floppy drive by looking at USB Mass Storage interface class
 			 * instead of Protocol: Uniform Floppy Interface (UFI) in /proc as we did before.
@@ -853,7 +888,6 @@ hotplug_event_begin_add_blockdev (const gchar *sysfs_path, const gchar *device_f
 			requires_eject = TRUE;
 		}
 
-
 		if (strcmp (hal_device_property_get_string (d, "storage.drive_type"), "floppy") == 0) {
 			no_partitions_hint = TRUE;
 		}
@@ -879,10 +913,10 @@ hotplug_event_begin_add_blockdev (const gchar *sysfs_path, const gchar *device_f
 		 * VOLUMES
 		 *
 		 ************************/
-
 		hal_device_property_set_string (d, "block.storage_device", parent->udi);
 
-		/* set defaults */
+		/* defaults */
+		hal_device_property_set_string (d, "storage.model", "");
 		hal_device_property_set_string (d, "volume.fstype", "");
 		hal_device_property_set_string (d, "volume.fsusage", "");
 		hal_device_property_set_string (d, "volume.fsversion", "");
@@ -911,9 +945,11 @@ hotplug_event_begin_add_blockdev (const gchar *sysfs_path, const gchar *device_f
 					partition_number = atoi (&sysfs_path[i+1]);
 					hal_device_property_set_int (d, "volume.partition.number", partition_number);
 				} else {
+					HAL_WARNING (("Cannot determine partition number?"));
 					goto error;
 				}
 			} else {
+				HAL_WARNING (("Cannot determine partition number"));
 				goto error;
 			}
 		}
@@ -928,9 +964,8 @@ hotplug_event_begin_add_blockdev (const gchar *sysfs_path, const gchar *device_f
 			goto error;
 		}
 		hal_device_property_set_uint64 (
-			d, "volume.size", 
-			((dbus_uint64_t)(512)) * 
-			((dbus_uint64_t)(hal_device_property_set_int (d, "volume.block_size", 512))));
+			d, "volume.size",
+			((dbus_uint64_t)(512)) * ((dbus_uint64_t)(hal_device_property_set_int (d, "volume.block_size", 512))));
 
 		/* add to TDL so preprobing callouts and prober can access it */
 		hal_device_store_add (hald_get_tdl (), d);
@@ -946,6 +981,7 @@ out2:
 	return;
 
 error:
+	HAL_WARNING (("Not adding device object"));
 	if (d != NULL)
 		g_object_unref (d);
 out:
@@ -954,86 +990,49 @@ out:
 }
 
 static void
-force_unmount (HalDevice *d)
+force_unmount_cb (HalDevice *d, guint32 exit_type, 
+		  gint return_code, gchar **error,
+		  gpointer data1, gpointer data2)
 {
-	const char *storudi;
-	HalDevice *stordev;
-	const char *device_file;
-	const char *device_mount_point;
-	const char *umount_argv[4] = { "/bin/umount", "-l", NULL, NULL };
-	char *umount_stdout;
-	char *umount_stderr;
-	int umount_exitcode;
+	void *end_token = (void *) data1;
 
-	device_file = hal_device_property_get_string (d, "block.device");
-	device_mount_point = hal_device_property_get_string (d, "volume.mount_point");
+	HAL_INFO (("force_unmount_cb for udi='%s', exit_type=%d, return_code=%d", d->udi, exit_type, return_code));
 
-	HAL_INFO (("Entering... udi=%s device_file=%s mount_point=%s", d->udi, device_file, device_mount_point));
+	if (exit_type == HALD_RUN_SUCCESS && error != NULL && 
+	    error[0] != NULL && error[1] != NULL) {
+		char *exp_name = NULL;
+		char *exp_detail = NULL;
 
-	/* Only attempt to 'umount -l' if some hal policy piece are performing policy on the device */
-	storudi = hal_device_property_get_string (d, "block.storage_device");
-	if (storudi == NULL) {
-		HAL_WARNING (("Could not get block.storage_device"));
-		goto out;
-	}
-	stordev = hal_device_store_find (hald_get_gdl (), storudi);
-	if (stordev == NULL) {
-		HAL_WARNING (("Could not get device object for storage device"));
-		goto out;
-	} else {
-		if ((!hal_device_has_property (stordev, "storage.policy.should_mount")) ||
-		    (!hal_device_property_get_bool (stordev, "storage.policy.should_mount"))) {
-			HAL_WARNING (("storage device doesn't have storage.policy.should_mount"));
-			goto out;
+		exp_name = error[0];
+		if (error[0] != NULL) {
+			exp_detail = error[1];
 		}
+		HAL_INFO (("failed with '%s' '%s'", exp_name, exp_detail));
 	}
 
-	umount_argv[2] = device_mount_point;
+	hal_util_callout_device_remove (d, blockdev_callouts_remove_done, end_token, NULL);
 
-	if (hal_device_has_property (d, "block.is_volume") &&
-	    hal_device_property_get_bool (d, "block.is_volume") &&
-	    hal_device_property_get_bool (d, "volume.is_mounted") &&
-	    device_mount_point != NULL &&
-	    strlen (device_mount_point) > 0) {
-		HAL_INFO (("attempting /bin/umount -l %s", device_mount_point));
+}
 
-		/* TODO: this is a bit dangerous; rather spawn async and do some timout on it */
+static void
+force_unmount (HalDevice *d, void *end_token)
+{
+	char *unmount_stdin;
+	char *extra_env[2];
 
-		/* invoke umount */
-		if (g_spawn_sync ("/",
-				  (char **) umount_argv,
-				  NULL,
-				  0,
-				  NULL,
-				  NULL,
-				  &umount_stdout,
-				  &umount_stderr,
-				  &umount_exitcode, NULL) != TRUE) {
-			HAL_ERROR (("Couldn't invoke /bin/umount"));
-		}
+	extra_env[0] = "HAL_METHOD_INVOKED_BY_UID=0";
+	extra_env[1] = NULL;
 
-		if (umount_exitcode != 0) {
-			HAL_ERROR (("/bin/umount returned %d", umount_exitcode));
-		} else {
-			/* Tell clients we are going to unmount so they close
-			 * can files - otherwise this unmount is going to stall
-			 *
-			 * One candidate for catching this would be FAM - the
-			 * File Alteration Monitor
-			 *
-			 * Lazy unmount been in Linux since 2.4.11, so we're
-			 * homefree (but other kernels might not support this)
-			 */
-			HAL_INFO (("Goint to emit VolumeUnmountForced('%s', '%s', TRUE)", device_file, device_mount_point));
-			device_send_signal_condition (d,
-						      "VolumeUnmountForced",
-						      device_file);
-		}
-	} else {
-		HAL_INFO (("Didn't want to unmount %s", device_file));
-	}
-out:
-	;
+	HAL_INFO (("force_unmount for udi='%s'", d->udi));
+
+	unmount_stdin = "lazy\n";
+
+	hald_runner_run_method (d, 
+				"hal-system-storage-unmount", extra_env, 
+				unmount_stdin, TRUE,
+				0,
+				force_unmount_cb,
+				end_token, NULL);
 }
 
 void
@@ -1052,6 +1051,8 @@ hotplug_event_begin_remove_blockdev (const gchar *sysfs_path, gboolean is_partit
 		HalDevice *stor_dev;
 		gboolean is_fakevolume;
 
+		is_partition = hal_device_property_get_bool (d, "volume.is_partition");
+
 		if (strcmp (hal_util_get_last_element (sysfs_path), "fakevolume") == 0)
 			is_fakevolume = TRUE;
 		else
@@ -1146,24 +1147,25 @@ hotplug_event_begin_remove_blockdev (const gchar *sysfs_path, gboolean is_partit
 
 		/* if we're mounted, then do a lazy unmount so the system can gracefully recover */
 		if (hal_device_property_get_bool (d, "volume.is_mounted")) {
-			force_unmount (d);
+			force_unmount (d, end_token);
+		} else {
+			hal_util_callout_device_remove (d, blockdev_callouts_remove_done, end_token, NULL);
 		}
-
-		hal_util_callout_device_remove (d, blockdev_callouts_remove_done, end_token, NULL);
 	}
 out:
 	;
 }
 
 static void 
-block_rescan_storage_done (HalDevice *d, gboolean timed_out, gint return_code, 
-			   gpointer data1, gpointer data2, HalHelperData *helper_data)
+block_rescan_storage_done (HalDevice *d, guint32 exit_type, 
+                           gint return_code, gchar **error,
+                           gpointer data1, gpointer data2)
 {
 	const char *sysfs_path;
 	HalDevice *fakevolume;
 	char fake_sysfs_path[HAL_PATH_MAX];
 
-	HAL_INFO (("hald-probe-storage --only-check-for-media returned %d (timed_out=%d)", return_code, timed_out));
+	HAL_INFO (("hald-probe-storage --only-check-for-media returned %d (exit_type=%d)", return_code, exit_type));
 
 	if (d == NULL) {
 		HAL_INFO (("Device object already removed"));
@@ -1214,13 +1216,11 @@ blockdev_rescan_device (HalDevice *d)
 	}
 
 	/* now see if we got a file system on the main block device */
-	if (hal_util_helper_invoke ("hald-probe-storage --only-check-for-media", NULL, d, NULL, 
-				    NULL, block_rescan_storage_done, 
-				    HAL_HELPER_TIMEOUT) == NULL) {
-		HAL_INFO (("Could not invoke 'hald-probe-storage --only-check-for-media'"));
-		goto out;
-	}
-	
+	hald_runner_run (d,
+	                 "hald-probe-storage --only-check-for-media", NULL, 
+	                 HAL_HELPER_TIMEOUT,
+	                 block_rescan_storage_done,
+	                 NULL, NULL);
 	ret = TRUE;
 
 out:
@@ -1233,20 +1233,35 @@ blockdev_generate_add_hotplug_event (HalDevice *d)
 {
 	const char *sysfs_path;
 	const char *device_file;
+	const char *model;
+	const char *vendor;
+	const char *serial;
+	const char *revision;
 	HotplugEvent *hotplug_event;
+	const char *nul;
+
+	nul = "\0";
 
 	sysfs_path = hal_device_property_get_string (d, "linux.sysfs_path");
+
 	device_file = hal_device_property_get_string (d, "block.device");
+	model       = hal_device_property_get_string (d, "storage.model");
+	vendor      = hal_device_property_get_string (d, "storage.vendor");
+	serial      = hal_device_property_get_string (d, "storage.serial");
+	revision    = hal_device_property_get_string (d, "storage.firmware_revision");
 
 	hotplug_event = g_new0 (HotplugEvent, 1);
 	hotplug_event->action = HOTPLUG_ACTION_ADD;
 	hotplug_event->type = HOTPLUG_EVENT_SYSFS;
 	g_strlcpy (hotplug_event->sysfs.subsystem, "block", sizeof (hotplug_event->sysfs.subsystem));
 	g_strlcpy (hotplug_event->sysfs.sysfs_path, sysfs_path, sizeof (hotplug_event->sysfs.sysfs_path));
-	if (device_file != NULL)
-		g_strlcpy (hotplug_event->sysfs.device_file, device_file, sizeof (hotplug_event->sysfs.device_file));
-	else
-		hotplug_event->sysfs.device_file[0] = '\0';
+
+	g_strlcpy (hotplug_event->sysfs.device_file, device_file != NULL ? device_file : nul, HAL_NAME_MAX);
+	g_strlcpy (hotplug_event->sysfs.vendor,           vendor != NULL ?      vendor : nul, HAL_NAME_MAX);
+	g_strlcpy (hotplug_event->sysfs.model,             model != NULL ?       model : nul, HAL_NAME_MAX);
+	g_strlcpy (hotplug_event->sysfs.serial,           serial != NULL ?      serial : nul, HAL_NAME_MAX);
+	g_strlcpy (hotplug_event->sysfs.revision,       revision != NULL ?    revision : nul, HAL_NAME_MAX);
+
 	hotplug_event->sysfs.net_ifindex = -1;
 
 	return hotplug_event;
