@@ -1,11 +1,11 @@
 /***************************************************************************
- * CVSID: $Id: ids.c,v 1.4 2005/02/28 20:56:50 david Exp $
+ * CVSID: $Id: ids.c,v 1.9 2005/11/02 15:38:14 david Exp $
  *
  * classdev.c : Handling of functional kernel devices
  *
  * Copyright (C) 2004 David Zeuthen, <david@fubar.dk>
  *
- * Licensed under the Academic Free License version 2.0
+ * Licensed under the Academic Free License version 2.1
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -19,7 +19,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  *
  **************************************************************************/
 
@@ -32,7 +32,7 @@
 #include <string.h>
 #include <mntent.h>
 #include <errno.h>
-#include <sys/types.h>
+#include <stdint.h>
 #include <sys/stat.h>
 #include <sys/socket.h>
 #include <sys/un.h>
@@ -115,10 +115,10 @@ pci_ids_line_iter_has_more ()
  *  @param  product_id          PCI product id or 0 if unknown
  *  @param  subsys_vendor_id    PCI subsystem vendor id or 0 if unknown
  *  @param  subsys_product_id   PCI subsystem product id or 0 if unknown
- *  @param  vendor_name         Set to pointer of result or #NULL
- *  @param  product_name        Set to pointer of result or #NULL
- *  @param  subsys_vendor_name  Set to pointer of result or #NULL
- *  @param  subsys_product_name Set to pointer of result or #NULL
+ *  @param  vendor_name         Set to pointer of result or NULL
+ *  @param  product_name        Set to pointer of result or NULL
+ *  @param  subsys_vendor_name  Set to pointer of result or NULL
+ *  @param  subsys_product_name Set to pointer of result or NULL
  */
 void
 ids_find_pci (int vendor_id, int product_id,
@@ -394,8 +394,8 @@ usb_ids_line_iter_has_more ()
  *
  *  @param  vendor_id           USB vendor id or 0 if unknown
  *  @param  product_id          USB product id or 0 if unknown
- *  @param  vendor_name         Set to pointer of result or #NULL
- *  @param  product_name        Set to pointer of result or #NULL
+ *  @param  vendor_name         Set to pointer of result or NULL
+ *  @param  product_name        Set to pointer of result or NULL
  */
 void
 ids_find_usb (int vendor_id, int product_id,
@@ -559,23 +559,37 @@ ids_init (void)
 }
 
 
-/* This, somewhat incomplete, list is from
+/* This, somewhat incomplete, list is from this sources:
  * http://www.plasma-online.de/english/identify/serial/pnp_id_pnp.html 
- * 
+ * http://www-pc.uni-regensburg.de/hardware/TECHNIK/PCI_PNP/pnpid.txt
+ *
  * Keep this sorted!
  */
 static char *pnp_ids_list[] = {
+	/* Crystal Semiconductor devices */
+	"CSC0000", "Crystal Semiconductor CS423x sound -- SB/WSS/OPL3 emulation",
+	"CSC0010", "Crystal Semiconductor CS423x sound -- control",
+	"CSC0001", "Crystal Semiconductor CS423x sound -- joystick",
+	"CSC0003", "Crystal Semiconductor CS423x sound -- MPU401",
+	/* IBM devices */
+	"IBM3780", "IBM pointing device",
+	"IBM0071", "IBM infrared communications device",
+	"IBM3760", "IBM DSP",
+	/* interrupt controllers */
 	"PNP0000", "AT Interrupt Controller",
 	"PNP0001", "EISA Interrupt Controller",
 	"PNP0002", "MCA Interrupt Controller",
 	"PNP0003", "APIC",
 	"PNP0004", "Cyrix SLiC MP interrupt controller",
+	/* timers */
 	"PNP0100", "AT Timer",
 	"PNP0101", "EISA Timer",
 	"PNP0102", "MCA Timer",
+	/* DMA controllers */
 	"PNP0200", "AT DMA Controller",
 	"PNP0201", "EISA DMA Controller",
 	"PNP0202", "MCA DMA Controller",
+	/* keyboards */
 	"PNP0300", "IBM PC/XT keyboard controller (83-key)",
 	"PNP0301", "IBM PC/AT keyboard controller (86-key)",
 	"PNP0302", "IBM PC/XT keyboard controller (84-key)",
@@ -602,18 +616,29 @@ static char *pnp_ids_list[] = {
 	"PNP0343", "Korean Enhanced keyboard 101b",
 	"PNP0343", "Korean Enhanced keyboard 101c",
 	"PNP0344", "Korean Enhanced keyboard 103",
+	/* parallel ports */
 	"PNP0400", "Standard LPT printer port",
 	"PNP0401", "ECP printer port",
+	/* serial ports */
 	"PNP0500", "Standard PC COM port",
 	"PNP0501", "16550A-compatible COM port",
+	"PNP0502", "Multiport serial device (non-intelligent 16550)",
 	"PNP0510", "Generic IRDA-compatible device",
+	"PNP0511", "Generic IRDA-compatible device",
+	/* IDE controller */
 	"PNP0600", "Generic ESDI/IDE/ATA compatible hard disk controller",
 	"PNP0601", "Plus Hardcard II",
 	"PNP0602", "Plus Hardcard IIXL/EZ",
 	"PNP0603", "Generic IDE supporting Microsoft Device Bay Specification",
+	"PNP0680", "Bus Master E-IDE controller",
+	/* floppy controllers */
+	"PNP0604", "PC standard floppy disk controller",
+	"PNP0605", "HP Omnibook floppy disk controller",
 	"PNP0700", "PC standard floppy disk controller",
 	"PNP0701", "Standard floppy controller supporting MS Device Bay Spec",
+	/* obsolete devices */
 	"PNP0802", "Microsoft Sound System compatible device (obsolete, use PNPB0xx instead)",
+	/* display adapters / graphic cards */
 	"PNP0900", "VGA Compatible",
 	"PNP0901", "Video Seven VRAM/VRAM II/1024i",
 	"PNP0902", "IBM 8514/A Compatible",
@@ -646,6 +671,7 @@ static char *pnp_ids_list[] = {
 	"PNP0940", "NCR 77c22e Super VGA",
 	"PNP0941", "NCR 77c32blt",
 	"PNP09FF", "Plug and Play Monitors (VESA DDC)",
+	/* peripheral buses */
 	"PNP0A00", "ISA Bus",
 	"PNP0A01", "EISA Bus",
 	"PNP0A02", "MCA Bus",
@@ -653,6 +679,7 @@ static char *pnp_ids_list[] = {
 	"PNP0A04", "VESA/VL Bus",
 	"PNP0A05", "Generic ACPI Bus",
 	"PNP0A06", "Generic ACPI Extended-IO Bus (EIO bus)",
+	/* system devices */
 	"PNP0800", "AT-style speaker sound",
 	"PNP0B00", "AT Real-Time Clock",
 	"PNP0C00", "Plug and Play BIOS (only created by the root enumerator)",
@@ -674,10 +701,15 @@ static char *pnp_ids_list[] = {
 	"PNP0C10", "ACPI system indicator device",
 	"PNP0C11", "ACPI thermal zone",
 	"PNP0C12", "Device Bay Controller",
+	"PNP0C13", "Plug and Play BIOS (used when ACPI mode cannot be used)",
+	"PNP0CF0", "Compaq LTE Lite Support",
+	"PNP0CF1", "Compaq LTE Elite Support",
+	/* PCMCIA controllers */
 	"PNP0E00", "Intel 82365-Compatible PCMCIA Controller",
 	"PNP0E01", "Cirrus Logic CL-PD6720 PCMCIA Controller",
 	"PNP0E02", "VLSI VL82C146 PCMCIA Controller",
 	"PNP0E03", "Intel 82365-compatible CardBus controller",
+	/* mice */
 	"PNP0F00", "Microsoft Bus Mouse",
 	"PNP0F01", "Microsoft Serial Mouse",
 	"PNP0F02", "Microsoft InPort Mouse",
@@ -715,6 +747,9 @@ static char *pnp_ids_list[] = {
 	"PNP0F22", "Reserved by Microsoft Input Device Group",
 	"PNP0F23", "Reserved by Microsoft Input Device Group",
 	"PNP0FFF", "Reserved by Microsoft Systems",
+	"PNP0XXX", "Unknown System Device",
+	/* network cards */
+	"PNP8000", "Network Adapter",
 	"PNP8001", "Novell/Anthem NE3200",
 	"PNP8004", "Compaq NE3200",
 	"PNP8006", "Intel EtherExpress/32",
@@ -722,8 +757,11 @@ static char *pnp_ids_list[] = {
 	"PNP8065", "Ungermann-Bass NIUps or NIUps/EOTP",
 	"PNP8072", "DEC (DE211) EtherWorks MC/TP",
 	"PNP8073", "DEC (DE212) EtherWorks MC/TP_BNC",
-	"PNP8078", "DCA 10 Mb MCA",
 	"PNP8074", "HP MC LAN Adapter/16 TP (PC27246)",
+	"PNP8078", "DCA 10 Mb MCA",
+	"PNP807F", "Racal NI9210",
+	"PNP8081", "Pure Data Ethernet",
+	"PNP8096", "Thomas-Conrad TC4046",
 	"PNP80C9", "IBM Token Ring",
 	"PNP80CA", "IBM Token Ring II",
 	"PNP80CB", "IBM Token Ring II/Short",
@@ -734,6 +772,7 @@ static char *pnp_ids_list[] = {
 	"PNP80D6", "NE2000 Compatible",
 	"PNP80D7", "Novell/Anthem NE1500T",
 	"PNP80D8", "Novell/Anthem NE2100",
+	"PNP80D9", "NE2000 Plus",
 	"PNP80DD", "SMC ARCNETPC",
 	"PNP80DE", "SMC ARCNET PC100, PC200",
 	"PNP80DF", "SMC ARCNET PC110, PC210, PC250",
@@ -762,11 +801,18 @@ static char *pnp_ids_list[] = {
 	"PNP80FE", "Thomas Conrad TC6145",
 	"PNP80FF", "Thomas Conrad TC6242",
 	"PNP8100", "Thomas Conrad TC6245",
+	"PNP8101", "Thomas-Conrad TC4045",
+	"PNP8104", "Thomas-Conrad TC4035",
 	"PNP8105", "DCA 10 MB",
 	"PNP8106", "DCA 10 MB Fiber Optic",
 	"PNP8107", "DCA 10 MB Twisted Pair",
 	"PNP8113", "Racal NI6510",
+	"PNP8114", "Racal NI5210/8 or NI5210/16",
+	"PNP8119", "Ungermann-Bass pcNIU",
+	"PNP811A", "Ungermann-Bass pcNIU/ex 128K",
+	"PNP811B", "Ungermann-Bass pcNIU/ex 512K", 
 	"PNP811C", "Ungermann-Bass NIUpc",
+	"PNP811D", "Ungermann-Bass NIUpc/3270",
 	"PNP8120", "Ungermann-Bass NIUpc/EOTP",
 	"PNP8123", "SMC StarCard PLUS (WD/8003S)",
 	"PNP8124", "SMC StarCard PLUS With On Board Hub (WD/8003SH)",
@@ -779,6 +825,7 @@ static char *pnp_ids_list[] = {
 	"PNP812F", "Intel TokenExpress 16/4",
 	"PNP8130", "Intel TokenExpress MCA 16/4",
 	"PNP8132", "Intel EtherExpress 16 (MCA)",
+	"PNP8133", "Compaq Ethernet 16E",
 	"PNP8137", "Artisoft AE-1",
 	"PNP8138", "Artisoft AE-2 or AE-3",
 	"PNP8141", "Amplicard AC 210/XT",
@@ -791,15 +838,24 @@ static char *pnp_ids_list[] = {
 	"PNP8159", "HP PC LAN Adapter/16 TL Plus (HP27252)",
 	"PNP815F", "National Semiconductor Ethernode *16AT",
 	"PNP8160", "National Semiconductor AT/LANTIC EtherNODE 16-AT3",
+	"PNP8169", "NCR StarCard",
 	"PNP816A", "NCR Token-Ring 4 Mbs ISA",
+	"PNP816B", "NCR WaveLAN AT",
+	"PNP816C", "NCR WaveLan MC",
 	"PNP816D", "NCR Token-Ring 16/4 Mbs ISA",
 	"PNP8191", "Olicom 16/4 Token-Ring Adapter",
+	"PNP81A5", "Research Machines Ethernet",
+	"PNP81B9", "ToshibaLAN (internal)",
 	"PNP81C3", "SMC EtherCard PLUS Elite (WD/8003EP)",
 	"PNP81C4", "SMC EtherCard PLUS 10T (WD/8003W)",
 	"PNP81C5", "SMC EtherCard PLUS Elite 16 (WD/8013EP)",
 	"PNP81C6", "SMC EtherCard PLUS Elite 16T (WD/8013W)",
 	"PNP81C7", "SMC EtherCard PLUS Elite 16 Combo (WD/8013EW or 8013EWC)",
 	"PNP81C8", "SMC EtherElite Ultra 16",
+	"PNP81C9", "SMC TigerCard (8216L, 8216LC, 8216LT)",
+	"PNP81CA", "SMC EtherEZ (8416)",
+	"PNP81D7", "Madge Smart 16/4 PC Ringnode",
+	"PNP81D8", "Madge Smart 16/4 Ringnode ISA",
 	"PNP81E4", "Pure Data PDI9025-32 (Token Ring)",
 	"PNP81E6", "Pure Data PDI508+ (ArcNet)",
 	"PNP81E7", "Pure Data PDI516+ (ArcNet)",
@@ -811,14 +867,18 @@ static char *pnp_ids_list[] = {
 	"PNP81F0", "Proteon Token Ring ISA (P1347)",
 	"PNP81FF", "Cabletron E2000 Series DNI",
 	"PNP8200", "Cabletron E2100 Series DNI",
+	"PNP8201", "Cabletron T2015 4/16 Mbit/s DNI",
 	"PNP8209", "Zenith Data Systems Z-Note",
 	"PNP820A", "Zenith Data Systems NE2000-Compatible",
 	"PNP8213", "Xircom Pocket Ethernet II",
 	"PNP8214", "Xircom Pocket Ethernet I",
+	"PNP8215", "Xircom Pocket Ethernet III Adapter",
 	"PNP821D", "RadiSys EXM-10",
 	"PNP8227", "SMC 3000 Series",
 	"PNP8228", "SMC 91C2 controller",
 	"PNP8231", "AMD AM2100/AM1500T",
+	"PNP824F", "RCE 10Base-T (16 bit)",
+	"PNP8250", "RCE 10Base-T (8 bit)",
 	"PNP8263", "Tulip NCC-16",
 	"PNP8277", "Exos 105",
 	"PNP828A", "Intel '595 based Ethernet",
@@ -827,17 +887,30 @@ static char *pnp_ids_list[] = {
 	"PNP828D", "AMD PCNet32 (VL version)",
 	"PNP8294", "IrDA Infrared NDIS driver (Microsoft-supplied)",
 	"PNP82BD", "IBM PCMCIA-NIC",
+	"PNP82C0", "Eagle Technology NE200T",
 	"PNP82C2", "Xircom CE10",
 	"PNP82C3", "Xircom CEM2",
+	"PNP82C4", "Xircom CE2",
 	"PNP8321", "DEC Ethernet (All Types)",
 	"PNP8323", "SMC EtherCard (All Types except 8013/A)",
 	"PNP8324", "ARCNET Compatible",
+	"PNP8325", "SMC TokenCard PLUS (8115T)",
 	"PNP8326", "Thomas Conrad (All Arcnet Types)",
 	"PNP8327", "IBM Token Ring (All Types)",
+	"PNP8328", "Ungermann-Bass NIU",
+	"PNP8329", "Proteon ProNET-4/16 ISA Token Ring (P1392+,P1392,1390)",
 	"PNP8385", "Remote Network Access [RNA] Driver",
 	"PNP8387", "Remote Network Access [RNA] PPP Driver",
 	"PNP8388", "Reserved for Microsoft Networking components",
 	"PNP8389", "Peer IrLAN infrared driver (Microsoft-supplied)",
+	"PNP8390", "Generic network adapter",
+	"PNP8XXX", "Unknown Network Adapter",
+	"PNPD300", "SK-NET TR4/16+ Token-Ring",
+	"PNPE000", "SK-NET G16, G16/TP Ethernet",
+	"PNPF000", "SK-NET FDDI-FI FDDI LAN",
+	/* CD controller */
+	"PNPA000", "Adaptec 154x compatible SCSI controller",
+	"PNPA001", "Adaptec 174x compatible SCSI controller",
 	"PNPA002", "Future Domain 16-700 compatible controller",
 	"PNPA003", "Mitsumi CD-ROM adapter (Panasonic spec., used on SBPro/SB16)",
 	"PNPA01B", "Trantor 128 SCSI Controller",
@@ -852,6 +925,8 @@ static char *pnp_ids_list[] = {
 	"PNPA030", "Mitsumi LU-005 Single Speed CD-ROM controller + drive",
 	"PNPA031", "Mitsumi FX-001 Single Speed CD-ROM controller + drive",
 	"PNPA032", "Mitsumi FX-001 Double Speed CD-ROM controller + drive",
+	"PNPAXXX", "Unknown SCSI, Proprietary CD Adapter",
+	/* multimedia devices */
 	"PNPB000", "Creative Labs Sound Blaster 1.5 (or compatible sound device)",
 	"PNPB001", "Creative Labs Sound Blaster 2.0 (or compatible sound device)",
 	"PNPB002", "Creative Labs Sound Blaster Pro (or compatible sound device)",
@@ -873,11 +948,23 @@ static char *pnp_ids_list[] = {
 	"PNPB019", "MediaVision Pro Audio Spectrum Basic (No Trantor SCSI, Thunder Chip Enabled)",
 	"PNPB020", "Yamaha OPL3-compatible FM synthesizer device",
 	"PNPB02F", "Joystick/Game port",
+	"PNPB077", "OAK Mozart Sound System",
+	"PNPB078", "OAK Mozart Sound System MPU-401",
+	"PNPBXXX", "Unknown Multimedia Device",
+	/* modems */
+	"PNP9000", "Modem",
 	"PNPC000", "Compaq 14400 Modem (TBD)",
 	"PNPC001", "Compaq 2400/9600 Modem (TBD)",
+	"PNPCXXX", "Unknown Modem",
+	/* Toshiba devices */
 	"TOS6200", "Toshiba Notebook Extra HCI driver",
 	"TOS6202", "Toshiba Notebook Extra HCI driver",
 	"TOS6207", "Toshiba Notebook Extra HCI driver",
+	"TOS7400", "Toshiba AcuPoint",
+	/* Wacom devices */
+	"WACf004", "Wacom Serial Tablet PC Pen Tablet/Digitizer",
+	"WACf005", "Wacom Serial Tablet PC Pen Tablet/Digitizer",
+	"WACf006", "Wacom Serial Tablet PC Pen Tablet/Digitizer",
 	NULL
 };
 
