@@ -1,11 +1,13 @@
 /***************************************************************************
- * CVSID: $Id: osspec.c,v 1.26 2005/06/27 19:13:03 david Exp $
+ * CVSID: $Id: osspec.c,v 1.45 2006/02/13 16:42:32 dkukawka Exp $
  *
  * osspec.c : New and improved HAL backend for Linux 2.6
  *
  * Copyright (C) 2004 David Zeuthen, <david@fubar.dk>
+ * Copyright (C) 2005,2006 Kay Sievers, <kay.sievers@vrfy.org>
+ * Copyright (C) 2005 Danny Kukawka, <danny.kukawka@web.de>
  *
- * Licensed under the Academic Free License version 2.0
+ * Licensed under the Academic Free License version 2.1
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -19,7 +21,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  *
  **************************************************************************/
 
@@ -44,21 +46,16 @@
 #include <sys/un.h>
 #include <sys/utsname.h>
 #include <unistd.h>
-#include <sys/types.h>
+#include <stdint.h>
 #include <sys/socket.h>
 #include <sys/stat.h>
 #include <sys/ioctl.h>
 #include <fcntl.h>
-#include <net/if_arp.h> /* for ARPHRD_... */
+#include <net/if_arp.h>
 #include <sys/socket.h>
 #include <linux/types.h>
-#include <linux/netlink.h>
-#include <linux/rtnetlink.h>
 #include <sys/ioctl.h>
 #include <net/if.h>
-
-#include <linux/netlink.h>
-#include <linux/rtnetlink.h>
 #include <sys/ioctl.h>
 #include <net/if.h>
 
@@ -70,25 +67,21 @@
 #include "../logger.h"
 #include "../hald.h"
 #include "../hald_dbus.h"
+#include "../hald_runner.h"
 #include "../device_info.h"
 #include "../util.h"
 
 #include "hotplug.h"
 #include "coldplug.h"
-
-#include "hotplug_helper.h"
-
 #include "ids.h"
-
 #include "acpi.h"
 #include "apm.h"
 #include "pmu.h"
 #include "blockdev.h"
-
 #include "osspec_linux.h"
 
-char hal_sysfs_path [HAL_PATH_MAX];
-char hal_proc_path [HAL_PATH_MAX];
+static char *hal_sysfs_path;
+static char *hal_proc_path;
 
 const gchar *
 get_hal_sysfs_path (void)
@@ -103,9 +96,8 @@ get_hal_proc_path (void)
 }
 
 static gboolean
-hald_helper_data (GIOChannel *source, GIOCondition condition, gpointer user_data)
+hald_udev_data (GIOChannel *source, GIOCondition condition, gpointer user_data)
 {
-	struct hald_helper_msg msg;
 	int fd;
 	int retval;
 	struct msghdr smsg;
@@ -114,10 +106,17 @@ hald_helper_data (GIOChannel *source, GIOCondition condition, gpointer user_data
 	struct ucred *cred;
 	char cred_msg[CMSG_SPACE(sizeof(struct ucred))];
 
+	char buf[4096];
+	size_t bufpos = 0;
+	const char *action = NULL;
+	HotplugEvent *hotplug_event;
+
+	memset(buf, 0x00, sizeof (buf));
+
 	fd = g_io_channel_unix_get_fd (source);
 
-	iov.iov_base = &msg;
-	iov.iov_len = sizeof (struct hald_helper_msg);
+	iov.iov_base = &buf;
+	iov.iov_len = sizeof (buf);
 
 	memset(&smsg, 0x00, sizeof (struct msghdr));
 	smsg.msg_iov = &iov;
@@ -144,246 +143,189 @@ hald_helper_data (GIOChannel *source, GIOCondition condition, gpointer user_data
 		goto out;
 	}
 
-	if (msg.magic != HALD_HELPER_MAGIC) {
-		HAL_INFO (("Magic is wrong, message ignored", cred->uid));
+	if (!strstr(buf, "@/")) {
+		HAL_INFO (("invalid message format"));
 		goto out;
 	}
 
-	HAL_INFO (("SEQNUM=%lld, TIMESTAMP=%d, ACTION=%s, SUBSYS=%s, SYSFSPATH=%s, DEVNAME=%s, IFINDEX=%d", 
-		   msg.seqnum, msg.time_stamp, msg.action, msg.subsystem, 
-		   msg.sysfs_path, msg.device_name, msg.net_ifindex));
+	hotplug_event = g_new0 (HotplugEvent, 1);
+	hotplug_event->type = HOTPLUG_EVENT_SYSFS;
 
-	if (strcmp (msg.action, "add") == 0) {
-		HotplugEvent *hotplug_event;
+	while (bufpos < sizeof (buf)) {
+		size_t keylen;
+		char *key;
+		char *str;
 
-		hotplug_event = g_new0 (HotplugEvent, 1);
-		hotplug_event->action = HOTPLUG_ACTION_ADD;
-		hotplug_event->type = HOTPLUG_EVENT_SYSFS;
-		g_strlcpy (hotplug_event->sysfs.subsystem, msg.subsystem, sizeof (hotplug_event->sysfs.subsystem));
-		g_snprintf (hotplug_event->sysfs.sysfs_path, sizeof (hotplug_event->sysfs.sysfs_path), "%s%s", 
-			    hal_sysfs_path, msg.sysfs_path);
-		g_strlcpy (hotplug_event->sysfs.device_file, msg.device_name, sizeof (hotplug_event->sysfs.device_file));
-		/* TODO: set wait_for_sysfs_path */
-		hotplug_event->sysfs.net_ifindex = msg.net_ifindex;
+		key = &buf[bufpos];
+		keylen = strlen(key);
+		if (keylen == 0)
+			break;
+		bufpos += keylen + 1;
 
-		/* queue up and process */
-		hotplug_event_enqueue (hotplug_event);
-		hotplug_event_process_queue ();
-
-	} else if (strcmp (msg.action, "remove") == 0) {
-		HotplugEvent *hotplug_event;
-
-		hotplug_event = g_new0 (HotplugEvent, 1);
-		hotplug_event->action = HOTPLUG_ACTION_REMOVE;
-		hotplug_event->type = HOTPLUG_EVENT_SYSFS;
-		g_strlcpy (hotplug_event->sysfs.subsystem, msg.subsystem, sizeof (hotplug_event->sysfs.subsystem));
-		g_snprintf (hotplug_event->sysfs.sysfs_path, sizeof (hotplug_event->sysfs.sysfs_path), "%s%s", 
-			    hal_sysfs_path, msg.sysfs_path);
-		g_strlcpy (hotplug_event->sysfs.device_file, msg.device_name, sizeof (hotplug_event->sysfs.device_file));
-		/* TODO: set wait_for_sysfs_path */
-		hotplug_event->sysfs.net_ifindex = msg.net_ifindex;
-
-		/* queue up and process */
-		hotplug_event_enqueue (hotplug_event);
-		hotplug_event_process_queue ();
+		if (strncmp(key, "ACTION=", 7) == 0)
+			action = &key[7];
+		else if (strncmp(key, "DEVPATH=", 8) == 0)
+			g_snprintf (hotplug_event->sysfs.sysfs_path, sizeof (hotplug_event->sysfs.sysfs_path),
+				    "%s%s", hal_sysfs_path, &key[8]);
+		else if (strncmp(key, "SUBSYSTEM=", 10) == 0)
+			g_strlcpy (hotplug_event->sysfs.subsystem, &key[10], sizeof (hotplug_event->sysfs.subsystem));
+		else if (strncmp(key, "DEVNAME=", 8) == 0)
+			g_strlcpy (hotplug_event->sysfs.device_file, &key[8], sizeof (hotplug_event->sysfs.device_file));
+		else if (strncmp(key, "SEQNUM=", 7) == 0)
+			hotplug_event->sysfs.seqnum = strtoull(&key[7], NULL, 10);
+		else if (strncmp(key, "IFINDEX=", 8) == 0)
+			hotplug_event->sysfs.net_ifindex = strtoul(&key[8], NULL, 10);
+		else if (strncmp(key, "ID_VENDOR=", 10) == 0) {
+			str = hal_util_strdup_valid_utf8(&key[10]);
+			g_strlcpy (hotplug_event->sysfs.vendor, str, sizeof(hotplug_event->sysfs.vendor));
+			g_free (str);
+		} else if (strncmp(key, "ID_MODEL=", 9) == 0) {
+			str = hal_util_strdup_valid_utf8(&key[9]);
+			g_strlcpy (hotplug_event->sysfs.model, str, sizeof(hotplug_event->sysfs.model));
+			g_free (str);
+		} else if (strncmp(key, "ID_REVISION=", 12) == 0) {
+			str = hal_util_strdup_valid_utf8(&key[12]);
+			g_strlcpy (hotplug_event->sysfs.revision, str, sizeof(hotplug_event->sysfs.revision));
+			g_free (str);
+		} else if (strncmp(key, "ID_SERIAL=", 10) == 0) {
+			str = hal_util_strdup_valid_utf8(&key[10]);
+			g_strlcpy (hotplug_event->sysfs.serial, str, sizeof(hotplug_event->sysfs.serial));
+			g_free (str);
+		} else if (strncmp(key, "ID_FS_USAGE=", 12) == 0) {
+			str = hal_util_strdup_valid_utf8(&key[12]);
+			g_strlcpy (hotplug_event->sysfs.fsusage, str, sizeof(hotplug_event->sysfs.fsusage));
+			g_free (str);
+		} else if (strncmp(key, "ID_FS_TYPE=", 11) == 0) {
+			str = hal_util_strdup_valid_utf8(&key[11]);
+			g_strlcpy (hotplug_event->sysfs.fstype, str, sizeof(hotplug_event->sysfs.fstype));
+			g_free (str);
+		} else if (strncmp(key, "ID_FS_VERSION=", 14) == 0) {
+			str = hal_util_strdup_valid_utf8(&key[14]);
+			g_strlcpy (hotplug_event->sysfs.fsversion, str, sizeof(hotplug_event->sysfs.fsversion));
+			g_free (str);
+		} else if (strncmp(key, "ID_FS_UUID=", 11) == 0) {
+			str = hal_util_strdup_valid_utf8(&key[11]);
+			g_strlcpy (hotplug_event->sysfs.fsuuid, str, sizeof(hotplug_event->sysfs.fsuuid));
+			g_free (str);
+		} else if (strncmp(key, "ID_FS_LABEL=", 12) == 0) {
+			str = hal_util_strdup_valid_utf8(&key[12]);
+			g_strlcpy (hotplug_event->sysfs.fslabel, str, sizeof(hotplug_event->sysfs.fslabel));
+			g_free (str);
+		}
 	}
 
+	if (!action) {
+		HAL_INFO (("missing ACTION"));
+		goto invalid;
+	}
+	if (hotplug_event->sysfs.sysfs_path == NULL) {
+		HAL_INFO (("missing DEVPATH"));
+		goto invalid;
+	}
+	if (hotplug_event->sysfs.subsystem == NULL) {
+		HAL_INFO (("missing SUSBSYSTEM"));
+		goto invalid;
+	}
+
+	HAL_INFO (("SEQNUM=%lld, ACTION=%s, SUBSYSTEM=%s, DEVPATH=%s, DEVNAME=%s, IFINDEX=%d",
+		   hotplug_event->sysfs.seqnum, action, hotplug_event->sysfs.subsystem, hotplug_event->sysfs.sysfs_path,
+		   hotplug_event->sysfs.device_file, hotplug_event->sysfs.net_ifindex));
+
+	if (strcmp (action, "add") == 0) {
+		hotplug_event->action = HOTPLUG_ACTION_ADD;
+		hotplug_event_enqueue (hotplug_event);
+		hotplug_event_process_queue ();
+		goto out;
+	}
+
+	if (strcmp (action, "remove") == 0) {
+		hotplug_event->action = HOTPLUG_ACTION_REMOVE;
+		hotplug_event_enqueue (hotplug_event);
+		hotplug_event_process_queue ();
+		goto out;
+	}
+
+invalid:
+	g_free (hotplug_event);
 
 out:
 	return TRUE;
 }
 
-#define VALID_NLMSG(h, s) ((NLMSG_OK (h, s) && \
-                           s >= sizeof (struct nlmsghdr) && \
-                           s >= h->nlmsg_len))
-
 static gboolean
-netlink_detection_data_ready (GIOChannel *channel, GIOCondition cond,
-			   gpointer user_data)
+mount_tree_changed_event (GIOChannel *channel, GIOCondition cond,
+		    gpointer user_data)
 {
-	int fd;
-	int bytes_read;
-	guint total_read = 0;
-	struct sockaddr_nl nladdr;
-	socklen_t nladdrlen = sizeof(nladdr);
-	char buf[1024];
-
-	if (cond & ~(G_IO_IN | G_IO_PRI)) {
-		HAL_ERROR (("Error occurred on netlink socket"));
+	if (cond & ~G_IO_ERR)
 		return TRUE;
-	}
 
-	fd = g_io_channel_unix_get_fd (channel);
-
-	do {
-		errno = 0;
-		bytes_read = recvfrom (fd,
-				   buf + total_read,
-				   sizeof (buf) - total_read,
-				   MSG_DONTWAIT,
-				   (struct sockaddr*)&nladdr, &nladdrlen);
-		if (nladdrlen != sizeof(nladdr)) {
-			HAL_ERROR(("Bad address size reading netlink socket"));
-			return TRUE;
-		}
-		if (nladdr.nl_pid) {
-			HAL_ERROR(("Spoofed packet received on netlink socket"));
-			return TRUE;
-		}
-		if (bytes_read > 0)
-			total_read += bytes_read;
-	} while (bytes_read > 0 || errno == EINTR);
-
-	if (bytes_read < 0 && errno != EAGAIN) {
-		HAL_ERROR (("Error reading data off netlink socket"));
-		return TRUE;
-	}
-
-	if (total_read > 0) {
-		HAL_INFO (("total_read=%d buf='%s'", total_read, buf));
-	}
-
-	/* Handle event: "mount@/block/hde" */
-	if (g_str_has_prefix (buf, "mount")) {
-		gchar sysfs_path[HAL_PATH_MAX];
-		g_strlcpy (sysfs_path, get_hal_sysfs_path (), sizeof (sysfs_path));
-		g_strlcat (sysfs_path, ((char *) buf) + sizeof ("mount"), sizeof (sysfs_path));
-		blockdev_mount_status_changed (sysfs_path, TRUE);
-	}
-
-	/* Handle event: "umount@/block/hde" */
-	if (g_str_has_prefix (buf, "umount")) {
-		gchar sysfs_path[HAL_PATH_MAX];
-		g_strlcpy (sysfs_path, get_hal_sysfs_path (), sizeof (sysfs_path));
-		g_strlcat (sysfs_path, ((char *) buf) + sizeof ("umount"), sizeof (sysfs_path));
-		blockdev_mount_status_changed (sysfs_path, FALSE);
-	}
+	HAL_INFO (("/proc/mounts tells, that the mount has tree changed"));
+	blockdev_refresh_mount_state (NULL);
 
 	return TRUE;
 }
 
-gboolean 
-hal_util_get_fs_mnt_path (const gchar *fs_type, gchar *mnt_path, gsize len)
-{
-	FILE *mnt;
-	struct mntent *mntent;
-	gboolean rc;
-	gsize dirlen;
-
-	rc = FALSE;
-	dirlen = 0;
-
-	if (fs_type == NULL || mnt_path == NULL || len == 0) {
-		HAL_ERROR (("Arguments not sane"));
-		return -1;
-	}
-
-	if ((mnt = setmntent ("/proc/mounts", "r")) == NULL) {
-		HAL_ERROR (("Error getting mount information"));
-		return -1;
-	}
-
-	while (rc == FALSE && dirlen == 0 && (mntent = getmntent(mnt)) != NULL) {
-		if (strcmp (mntent->mnt_type, fs_type) == 0) {
-			dirlen = strlen (mntent->mnt_dir);
-			if (dirlen <= (len - 1)) {
-				g_strlcpy (mnt_path, mntent->mnt_dir, len);
-				rc = TRUE;
-			} else {
-				HAL_ERROR (("Error - mount path too long"));
-				rc = FALSE;
-			}
-		}
-	}
-	endmntent (mnt);
-	
-	if (dirlen == 0 && rc == TRUE) {
-		HAL_ERROR (("Filesystem %s not found", fs_type));
-		rc = FALSE;
-	}
-
-	if ((!hal_util_remove_trailing_slash (mnt_path)))
-		rc = FALSE;
-	
-	return rc;
-}
-
 void
 osspec_init (void)
 {
-	int socketfd;
+	gchar path[HAL_PATH_MAX];
+	int udev_socket;
 	struct sockaddr_un saddr;
 	socklen_t addrlen;
-	GIOChannel *channel;	
 	const int on = 1;
-	static int netlink_fd = -1;
-	struct sockaddr_nl netlink_addr;
-	GIOChannel *netlink_channel;
+	GIOChannel *udev_channel;
+	GIOChannel *mounts_channel;
 
-	/* setup socket for listening from datagrams from the hal.hotplug helper */
+	/*
+	 * setup socket for listening from messages from udev
+	 */
 	memset(&saddr, 0x00, sizeof(saddr));
 	saddr.sun_family = AF_LOCAL;
 	/* use abstract namespace for socket path */
-	strcpy(&saddr.sun_path[1], HALD_HELPER_SOCKET_PATH);
+	strcpy(&saddr.sun_path[1], "/org/freedesktop/hal/udev_event");
 	addrlen = offsetof(struct sockaddr_un, sun_path) + strlen(saddr.sun_path+1) + 1;
 
-	socketfd = socket(AF_LOCAL, SOCK_DGRAM, 0);
-	if (socketfd == -1) {
+	udev_socket = socket(AF_LOCAL, SOCK_DGRAM, 0);
+	if (udev_socket == -1) {
 		DIE (("Couldn't open socket"));
 	}
 
-	if (bind(socketfd, (struct sockaddr *) &saddr, addrlen) < 0) {
-		fprintf (stderr, "Error binding to %s: %s\n", HALD_HELPER_SOCKET_PATH, strerror(errno));
+	if (bind(udev_socket, (struct sockaddr *) &saddr, addrlen) < 0) {
+		fprintf (stderr, "Error binding udev_event socket: %s\n", strerror(errno));
 		exit (1);
 	}
 	/* enable receiving of the sender credentials */
-	setsockopt(socketfd, SOL_SOCKET, SO_PASSCRED, &on, sizeof(on));
+	setsockopt(udev_socket, SOL_SOCKET, SO_PASSCRED, &on, sizeof(on));
 
-	channel = g_io_channel_unix_new (socketfd);
-	g_io_add_watch (channel, G_IO_IN, hald_helper_data, NULL);
-	g_io_channel_unref (channel);
+	udev_channel = g_io_channel_unix_new (udev_socket);
+	g_io_add_watch (udev_channel, G_IO_IN, hald_udev_data, NULL);
+	g_io_channel_unref (udev_channel);
 
-	/* Get mount points for /proc and /sys */
-	if (!hal_util_get_fs_mnt_path ("sysfs", hal_sysfs_path, sizeof (hal_sysfs_path))) {
-		HAL_ERROR (("Could not get sysfs mount point"));
-		goto error;
-	}
-	HAL_INFO (("sysfs mount point is '%s'", hal_sysfs_path));
-	if (!hal_util_get_fs_mnt_path ("proc", hal_proc_path, sizeof (hal_proc_path))) {
-		HAL_ERROR (("Could not get proc mount point"));
-		goto error;
-	}
-	HAL_INFO (("proc mount point is '%s'", hal_proc_path));
-
-	/* hook up to netlink socket to receive events from the Kernel Events
-	 * Layer (available since 2.6.10) - TODO: Don't use the constant 15 but
-	 * rather the NETLINK_KOBJECT_UEVENT symbol
+	/*
+	 * set mount points for /proc and /sys, possibly overridden for testing
 	 */
-	netlink_fd = socket (PF_NETLINK, SOCK_DGRAM, 15/*NETLINK_KOBJECT_UEVENT*/);
+	hal_sysfs_path = getenv ("SYSFS_PATH");
+	if (hal_sysfs_path == NULL)
+		hal_sysfs_path = "/sys";
 
-	if (netlink_fd < 0) {
-		DIE (("Unable to create netlink socket"));
-	}
+	hal_proc_path = getenv ("PROC_PATH");
+	if (hal_proc_path == NULL)
+		hal_proc_path = "/proc";
 
-	memset (&netlink_addr, 0, sizeof (netlink_addr));
-	netlink_addr.nl_family = AF_NETLINK;
-	netlink_addr.nl_pid = getpid ();
-	netlink_addr.nl_groups = 0xffffffff;//RTMGRP_LINK;//1 << 15 /*NETLINK_KOBJECT_UEVENT*/;
+	/*
+	 * watch /proc/mounts for mount tree changes
+	 * kernel 2.6.15 vfs throws a POLLERR event for every change
+	 */
+	g_snprintf (path, sizeof (path), "%s/mounts", get_hal_proc_path ());
+	mounts_channel = g_io_channel_new_file (path, "r", NULL);
+	if (mounts_channel == NULL)
+		DIE (("Unable to read /proc/mounts"));
+	g_io_add_watch (mounts_channel, G_IO_ERR, mount_tree_changed_event, NULL);
 
-	if (bind (netlink_fd, (struct sockaddr *) &netlink_addr, sizeof (netlink_addr)) < 0) {
-		DIE (("Unable to bind to netlink socket"));
-	}
-
-	netlink_channel = g_io_channel_unix_new (netlink_fd);
-
-	g_io_add_watch (netlink_channel, G_IO_IN | G_IO_PRI | G_IO_ERR | G_IO_NVAL,
-			netlink_detection_data_ready, NULL);
-
-	/* Load various hardware id databases */
+	/*
+	 *Load various hardware id databases
+	 */
 	ids_init ();
-
-error:
-	;
 }
 
 static void 
@@ -417,8 +359,9 @@ computer_probing_helper_done (HalDevice *d)
 }
 
 static void 
-computer_probing_pcbios_helper_done (HalDevice *d, gboolean timed_out, gint return_code, 
-				     gpointer data1, gpointer data2, HalHelperData *helper_data)
+computer_probing_pcbios_helper_done (HalDevice *d, guint32 exit_type, 
+		                                 gint return_code, gchar **error, 
+				                             gpointer data1, gpointer data2)
 {
 	const char *chassis_type;
 	const char *system_manufacturer;
@@ -430,9 +373,14 @@ computer_probing_pcbios_helper_done (HalDevice *d, gboolean timed_out, gint retu
 	    (system_version = hal_device_property_get_string (d, "smbios.system.version")) != NULL) {
 		char buf[128];
 
-		g_snprintf (buf, sizeof (buf), "%s %s", system_product, system_version);
 		hal_device_property_set_string (d, "system.vendor", system_manufacturer);
-		hal_device_property_set_string (d, "system.product", buf);
+
+		if (strcmp(system_version, "Not Specified" ) != 0 ) {
+			g_snprintf (buf, sizeof (buf), "%s %s", system_product, system_version);
+			hal_device_property_set_string (d, "system.product", buf);
+		} else {
+			hal_device_property_set_string (d, "system.product", system_product);
+		}
 	}
 
 
@@ -478,7 +426,9 @@ computer_probing_pcbios_helper_done (HalDevice *d, gboolean timed_out, gint retu
 
 		for (i = 0; chassis_map[i] != NULL; i += 2) {
 			if (strcmp (chassis_map[i], chassis_type) == 0) {
-				hal_device_property_set_string (d, "system.formfactor", chassis_map[i+1]);
+				/* check if the key is 'unknown' to prevent overwrite keys */
+				if (strcmp (hal_device_property_get_string (d, "system.formfactor"), "unknown"))
+					hal_device_property_set_string (d, "system.formfactor", chassis_map[i+1]);
 				break;
 			}
 		}
@@ -488,6 +438,49 @@ computer_probing_pcbios_helper_done (HalDevice *d, gboolean timed_out, gint retu
 	computer_probing_helper_done (d);
 }
 
+static void
+set_suspend_hibernate_keys (HalDevice *d)
+{
+	int can_suspend;
+	int can_hibernate;
+	ssize_t read;
+	unsigned int len;
+	char *poweroptions;
+	FILE *fp;
+
+	can_suspend = FALSE;
+	can_hibernate = FALSE;
+
+	/* try to find 'mem' and 'disk' in /sys/power/state */
+	fp = fopen ("/sys/power/state", "r");
+	if (fp == NULL) {
+		HAL_WARNING (("Could not open /sys/power/state"));
+		goto out;
+	}
+	poweroptions = NULL;
+	len = 0;
+	read = getline (&poweroptions, &len, fp);
+	fclose (fp);
+	if (poweroptions == NULL) {
+		HAL_WARNING (("Contents of /sys/power/state invalid"));
+		goto out;
+	}
+	if (strstr (poweroptions, "mem"))
+		can_suspend = TRUE;
+	if (strstr (poweroptions, "disk"))
+		can_hibernate = TRUE;
+	free (poweroptions);
+
+	/* check for the presence of suspend2 */
+	if (access ("/proc/software_suspend", F_OK) == 0)
+		can_hibernate = TRUE;
+	if (access ("/proc/suspend2", F_OK) == 0)
+		can_hibernate = TRUE;
+out:
+	hal_device_property_set_bool (d, "power_management.can_suspend_to_ram", can_suspend);
+	hal_device_property_set_bool (d, "power_management.can_suspend_to_disk", can_hibernate);
+}
+
 void 
 osspec_probe (void)
 {
@@ -535,19 +528,25 @@ osspec_probe (void)
 	}
 	HAL_INFO (("Done synthesizing events"));
 
+	/*
+	 * Populate the powermgmt keys according to the kernel options.
+	 * NOTE: This may not mean the machine is able to suspend
+	 *	 or hibernate successfully, only that the machine has
+	 *	 support compiled into the kernel.
+	 */
+	set_suspend_hibernate_keys (root);
+
 	/* TODO: add prober for PowerMac's */
-
 	if (should_decode_dmi) {
-		if (hal_util_helper_invoke ("hald-probe-smbios", NULL, root, NULL, NULL,
-					    computer_probing_pcbios_helper_done, 
-					    HAL_HELPER_TIMEOUT) != NULL)
-			goto out;
-	}
+		hald_runner_run (root, "hald-probe-smbios", NULL,
+                         HAL_HELPER_TIMEOUT,
+                         computer_probing_pcbios_helper_done, 
+                         NULL, NULL);
+	} else {
+		/* no probing */
+		computer_probing_helper_done (root);
+  }
 
-	/* no probing or probing failed */
-	computer_probing_helper_done (root);
-out:
-	;
 }
 
 DBusHandlerResult
