@@ -1,5 +1,5 @@
 /***************************************************************************
- * CVSID: $Id: libhal-storage.c,v 1.16 2005/06/28 15:47:51 david Exp $
+ * CVSID: $Id: libhal-storage.c,v 1.32 2006/02/23 00:23:13 david Exp $
  *
  * libhal-storage.c : HAL convenience library for storage devices and volumes
  *
@@ -7,7 +7,7 @@
  *
  * Author: David Zeuthen <davidz@redhat.com>
  *
- * Licensed under the Academic Free License version 2.0
+ * Licensed under the Academic Free License version 2.1
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -64,6 +64,10 @@
  *  @{
  */
 
+/**
+ * @typedef typedef struct IconMappingEntry_s {} IconMappingEntry
+ * @brief typdef of IconMappingEntry_s
+ */
 typedef struct IconMappingEntry_s {
 	LibHalStoragePolicyIcon icon;
 	char *path;
@@ -277,12 +281,31 @@ libhal_drive_policy_compute_display_name (LibHalDrive *drive, LibHalVolume *volu
 		if (drive_cdrom_caps & LIBHAL_DRIVE_CDROM_CAPS_DVDRAM)
 			second = "/DVD-RAM";
 		if ((drive_cdrom_caps & LIBHAL_DRIVE_CDROM_CAPS_DVDR) &&
-		    (drive_cdrom_caps & LIBHAL_DRIVE_CDROM_CAPS_DVDPLUSR))
-			second = "/DVD±R";
+		    (drive_cdrom_caps & LIBHAL_DRIVE_CDROM_CAPS_DVDPLUSR)) {
+			if(drive_cdrom_caps & LIBHAL_DRIVE_CDROM_CAPS_DVDPLUSRDL)
+				second = "/DVD±R DL";
+			else
+				second = "/DVD±R";
+		}
 		if ((drive_cdrom_caps & LIBHAL_DRIVE_CDROM_CAPS_DVDRW) &&
-		    (drive_cdrom_caps & LIBHAL_DRIVE_CDROM_CAPS_DVDPLUSRW))
-			second = "/DVD±RW";
-
+		    (drive_cdrom_caps & LIBHAL_DRIVE_CDROM_CAPS_DVDPLUSRW)) {
+                        if(drive_cdrom_caps & LIBHAL_DRIVE_CDROM_CAPS_DVDPLUSRDL)
+                                second = "/DVD±RW DL";
+                        else
+                                second = "/DVD±RW";
+                }
+		if (drive_cdrom_caps & LIBHAL_DRIVE_CDROM_CAPS_BDROM)
+			second = "/BD-ROM";
+		if (drive_cdrom_caps & LIBHAL_DRIVE_CDROM_CAPS_BDR)
+			second = "/BD-R";
+		if (drive_cdrom_caps & LIBHAL_DRIVE_CDROM_CAPS_BDRE)
+			second = "/BD-RE";
+		if (drive_cdrom_caps & LIBHAL_DRIVE_CDROM_CAPS_HDDVDROM)
+			second = "/HD DVD-ROM";
+		if (drive_cdrom_caps & LIBHAL_DRIVE_CDROM_CAPS_HDDVDR)
+			second = "/HD DVD-R";
+		if (drive_cdrom_caps & LIBHAL_DRIVE_CDROM_CAPS_HDDVDRW)
+			second = "/HD DVD-RW";
 
 		if (drive_is_hotpluggable) {
 			snprintf (buf, MAX_STRING_SZ, _("External %s%s Drive"), first, second);
@@ -374,68 +397,111 @@ libhal_volume_policy_compute_display_name (LibHalDrive *drive, LibHalVolume *vol
 		default:
 			/* explict fallthrough */
 		case LIBHAL_VOLUME_DISC_TYPE_CDROM:
-			name = strdup (_("CD-ROM Disc"));
+			name = strdup (_("CD-ROM "));
 			break;
 			
 		case LIBHAL_VOLUME_DISC_TYPE_CDR:
 			if (libhal_volume_disc_is_blank (volume))
-				name = strdup (_("Blank CD-R Disc"));
+				name = strdup (_("Blank CD-R"));
 			else
-				name = strdup (_("CD-R Disc"));
+				name = strdup (_("CD-R"));
 			break;
 			
 		case LIBHAL_VOLUME_DISC_TYPE_CDRW:
 			if (libhal_volume_disc_is_blank (volume))
-				name = strdup (_("Blank CD-RW Disc"));
+				name = strdup (_("Blank CD-RW"));
 			else
-				name = strdup (_("CD-RW Disc"));
+				name = strdup (_("CD-RW"));
 			break;
 			
 		case LIBHAL_VOLUME_DISC_TYPE_DVDROM:
-			name = strdup (_("DVD-ROM Disc"));
+			name = strdup (_("DVD-ROM"));
 			break;
 			
 		case LIBHAL_VOLUME_DISC_TYPE_DVDRAM:
 			if (libhal_volume_disc_is_blank (volume))
-				name = strdup (_("Blank DVD-RAM Disc"));
+				name = strdup (_("Blank DVD-RAM"));
 			else
-				name = strdup (_("DVD-RAM Disc"));
+				name = strdup (_("DVD-RAM"));
 			break;
 			
 		case LIBHAL_VOLUME_DISC_TYPE_DVDR:
 			if (libhal_volume_disc_is_blank (volume))
-				name = strdup (_("Blank DVD-R Disc"));
+				name = strdup (_("Blank DVD-R"));
 			else
-				name = strdup (_("DVD-R Disc"));
+				name = strdup (_("DVD-R"));
 			break;
 			
 		case LIBHAL_VOLUME_DISC_TYPE_DVDRW:
 			if (libhal_volume_disc_is_blank (volume))
-				name = strdup (_("Blank DVD-RW Disc"));
+				name = strdup (_("Blank DVD-RW"));
 			else
-				name = strdup (_("DVD-RW Disc"));
-
+				name = strdup (_("DVD-RW"));
 			break;
 
 		case LIBHAL_VOLUME_DISC_TYPE_DVDPLUSR:
 			if (libhal_volume_disc_is_blank (volume))
-				name = strdup (_("Blank DVD+R Disc"));
+				name = strdup (_("Blank DVD+R"));
 			else
-				name = strdup (_("DVD+R Disc"));
+				name = strdup (_("DVD+R"));
 			break;
 			
 		case LIBHAL_VOLUME_DISC_TYPE_DVDPLUSRW:
 			if (libhal_volume_disc_is_blank (volume))
-				name = strdup (_("Blank DVD+RW Disc"));
+				name = strdup (_("Blank DVD+RW"));
 			else
-				name = strdup (_("DVD+RW Disc"));
+				name = strdup (_("DVD+RW"));
 			break;
+		
+		case LIBHAL_VOLUME_DISC_TYPE_DVDPLUSR_DL:
+			if (libhal_volume_disc_is_blank (volume))
+				name = strdup (_("Blank DVD+R Dual-Layer"));
+			else
+				name = strdup (_("DVD+R Dual-Layer"));
+			break;
+		
+		case LIBHAL_VOLUME_DISC_TYPE_BDROM:
+			name = strdup (_("BD-ROM"));
+			break;
+			
+		case LIBHAL_VOLUME_DISC_TYPE_BDR:
+			if (libhal_volume_disc_is_blank (volume))
+				name = strdup (_("Blank BD-R"));
+			else
+				name = strdup (_("BD-R"));
+			break;
+		
+		case LIBHAL_VOLUME_DISC_TYPE_BDRE:
+			if (libhal_volume_disc_is_blank (volume))
+				name = strdup (_("Blank BD-RE"));
+			else
+				name = strdup (_("BD-RE"));
+			break;
+		
+		case LIBHAL_VOLUME_DISC_TYPE_HDDVDROM:
+			name = strdup (_("HD DVD-ROM"));
+			break;
+			
+		case LIBHAL_VOLUME_DISC_TYPE_HDDVDR:
+			if (libhal_volume_disc_is_blank (volume))
+				name = strdup (_("Blank HD DVD-R"));
+			else
+				name = strdup (_("HD DVD-R"));
+			break;
+			
+		case LIBHAL_VOLUME_DISC_TYPE_HDDVDRW:
+			if (libhal_volume_disc_is_blank (volume))
+				name = strdup (_("Blank HD DVD-RW"));
+			else
+				name = strdup (_("HD DVD-RW"));
+			break;
+
 		}
 		
 		/* Special case for pure audio disc */
 		if (libhal_volume_disc_has_audio (volume) && !libhal_volume_disc_has_data (volume)) {
 			free (name);
-			name = strdup (_("Audio Disc"));
+			name = strdup (_("Audio CD"));
 		}
 
 		goto out;
@@ -696,6 +762,10 @@ struct LibHalVolume_s {
 	char *mount_filesystem;
 	dbus_bool_t should_mount;
 
+	dbus_bool_t ignore_volume;
+
+	char *crypto_backing_volume;
+
 	char mount_options[MOUNT_OPTIONS_SIZE];
 };
 
@@ -732,12 +802,14 @@ libhal_drive_free (LibHalDrive *drive)
 	libhal_free_string (drive->desired_mount_point);
 	libhal_free_string (drive->mount_filesystem);
 	libhal_free_string_array (drive->capabilities);
+
+	free (drive);
 }
 
 
 /** Free all resources used by a LibHalVolume object.
  *
- *  @param  volume              Object to free
+ *  @param  vol              Object to free
  */
 void
 libhal_volume_free (LibHalVolume *vol)
@@ -754,6 +826,9 @@ libhal_volume_free (LibHalVolume *vol)
 	libhal_free_string (vol->uuid);
 	libhal_free_string (vol->desired_mount_point);
 	libhal_free_string (vol->mount_filesystem);
+	libhal_free_string (vol->crypto_backing_volume);
+
+	free (vol);
 }
 
 
@@ -799,7 +874,7 @@ out:
  */
 LibHalDrive *
 libhal_drive_from_udi (LibHalContext *hal_ctx, const char *udi)
-{
+{	
 	char *bus_textual;
 	LibHalDrive *drive;
 	LibHalPropertySet *properties;
@@ -807,6 +882,8 @@ libhal_drive_from_udi (LibHalContext *hal_ctx, const char *udi)
 	DBusError error;
 	unsigned int i;
 
+	LIBHAL_CHECK_LIBHALCONTEXT(hal_ctx, NULL);
+
 	drive = NULL;
 	properties = NULL;
 	bus_textual = NULL;
@@ -865,9 +942,16 @@ libhal_drive_from_udi (LibHalContext *hal_ctx, const char *udi)
 		LIBHAL_PROP_EXTRACT_BOOL_BITFIELD ("storage.cdrom.dvd", drive->cdrom_caps, LIBHAL_DRIVE_CDROM_CAPS_DVDROM);
 		LIBHAL_PROP_EXTRACT_BOOL_BITFIELD ("storage.cdrom.dvdplusr", drive->cdrom_caps, LIBHAL_DRIVE_CDROM_CAPS_DVDPLUSR);
 		LIBHAL_PROP_EXTRACT_BOOL_BITFIELD ("storage.cdrom.dvdplusrw", drive->cdrom_caps, LIBHAL_DRIVE_CDROM_CAPS_DVDPLUSRW);
+		LIBHAL_PROP_EXTRACT_BOOL_BITFIELD ("storage.cdrom.dvdplusrdl", drive->cdrom_caps, LIBHAL_DRIVE_CDROM_CAPS_DVDPLUSRDL);
 		LIBHAL_PROP_EXTRACT_BOOL_BITFIELD ("storage.cdrom.dvdr", drive->cdrom_caps, LIBHAL_DRIVE_CDROM_CAPS_DVDR);
 		LIBHAL_PROP_EXTRACT_BOOL_BITFIELD ("storage.cdrom.dvdrw", drive->cdrom_caps, LIBHAL_DRIVE_CDROM_CAPS_DVDRW);
 		LIBHAL_PROP_EXTRACT_BOOL_BITFIELD ("storage.cdrom.dvdram", drive->cdrom_caps, LIBHAL_DRIVE_CDROM_CAPS_DVDRAM);
+		LIBHAL_PROP_EXTRACT_BOOL_BITFIELD ("storage.cdrom.bd", drive->cdrom_caps, LIBHAL_DRIVE_CDROM_CAPS_BDROM);
+		LIBHAL_PROP_EXTRACT_BOOL_BITFIELD ("storage.cdrom.bdr", drive->cdrom_caps, LIBHAL_DRIVE_CDROM_CAPS_BDR);
+		LIBHAL_PROP_EXTRACT_BOOL_BITFIELD ("storage.cdrom.bdre", drive->cdrom_caps, LIBHAL_DRIVE_CDROM_CAPS_BDRE);
+		LIBHAL_PROP_EXTRACT_BOOL_BITFIELD ("storage.cdrom.hddvd", drive->cdrom_caps, LIBHAL_DRIVE_CDROM_CAPS_HDDVDROM);
+		LIBHAL_PROP_EXTRACT_BOOL_BITFIELD ("storage.cdrom.hddvdr", drive->cdrom_caps, LIBHAL_DRIVE_CDROM_CAPS_HDDVDR);
+		LIBHAL_PROP_EXTRACT_BOOL_BITFIELD ("storage.cdrom.hddvdrw", drive->cdrom_caps, LIBHAL_DRIVE_CDROM_CAPS_HDDVDRW);
 
 		LIBHAL_PROP_EXTRACT_BOOL   ("storage.policy.should_mount",        drive->should_mount);
 		LIBHAL_PROP_EXTRACT_STRING ("storage.policy.desired_mount_point", drive->desired_mount_point);
@@ -980,14 +1064,18 @@ LibHalVolume *
 libhal_volume_from_udi (LibHalContext *hal_ctx, const char *udi)
 {
 	char *disc_type_textual;
+	char *vol_fsusage_textual;
 	LibHalVolume *vol;
 	LibHalPropertySet *properties;
 	LibHalPropertySetIterator it;
 	DBusError error;
 
+	LIBHAL_CHECK_LIBHALCONTEXT(hal_ctx, NULL);
+
 	vol = NULL;
 	properties = NULL;
 	disc_type_textual = NULL;
+	vol_fsusage_textual = NULL;
 
 	dbus_error_init (&error);
 	if (!libhal_device_query_capability (hal_ctx, udi, "volume", &error))
@@ -1022,12 +1110,19 @@ libhal_volume_from_udi (LibHalContext *hal_ctx, const char *udi)
 
 		LIBHAL_PROP_EXTRACT_STRING ("block.storage_device",      vol->storage_device);
 
+		LIBHAL_PROP_EXTRACT_STRING ("volume.crypto_luks.clear.backing_volume", vol->crypto_backing_volume);
+
 		LIBHAL_PROP_EXTRACT_INT    ("volume.block_size",         vol->block_size);
 		LIBHAL_PROP_EXTRACT_INT    ("volume.num_blocks",         vol->num_blocks);
 		LIBHAL_PROP_EXTRACT_STRING ("volume.label",              vol->volume_label);
 		LIBHAL_PROP_EXTRACT_STRING ("volume.mount_point",        vol->mount_point);
 		LIBHAL_PROP_EXTRACT_STRING ("volume.fstype",             vol->fstype);
+		LIBHAL_PROP_EXTRACT_STRING ("volume.fsversion",             vol->fsversion);
 		LIBHAL_PROP_EXTRACT_BOOL   ("volume.is_mounted",         vol->is_mounted);
+		LIBHAL_PROP_EXTRACT_STRING ("volume.fsusage",            vol_fsusage_textual);
+		LIBHAL_PROP_EXTRACT_STRING ("volume.uuid",               vol->uuid);
+
+		LIBHAL_PROP_EXTRACT_BOOL   ("volume.ignore",             vol->ignore_volume);
 
 		LIBHAL_PROP_EXTRACT_BOOL   ("volume.is_disc",            vol->is_disc);
 		LIBHAL_PROP_EXTRACT_STRING ("volume.disc.type",          disc_type_textual);
@@ -1065,13 +1160,42 @@ libhal_volume_from_udi (LibHalContext *hal_ctx, const char *udi)
 			vol->disc_type = LIBHAL_VOLUME_DISC_TYPE_DVDPLUSRW;
 		} else if (strcmp (disc_type_textual, "dvd_plus_r_dl") == 0) {
 			vol->disc_type = LIBHAL_VOLUME_DISC_TYPE_DVDPLUSR_DL;
+		} else if (strcmp (disc_type_textual, "bd_rom") == 0) {
+			vol->disc_type = LIBHAL_VOLUME_DISC_TYPE_BDROM;
+		} else if (strcmp (disc_type_textual, "bd_r") == 0) {
+			vol->disc_type = LIBHAL_VOLUME_DISC_TYPE_BDR;
+		} else if (strcmp (disc_type_textual, "bd_re") == 0) {
+			vol->disc_type = LIBHAL_VOLUME_DISC_TYPE_BDRE;
+		} else if (strcmp (disc_type_textual, "hddvd_rom") == 0) {
+			vol->disc_type = LIBHAL_VOLUME_DISC_TYPE_HDDVDROM;
+		} else if (strcmp (disc_type_textual, "hddvd_r") == 0) {
+			vol->disc_type = LIBHAL_VOLUME_DISC_TYPE_HDDVDR;
+		} else if (strcmp (disc_type_textual, "hddvd_rw") == 0) {
+			vol->disc_type = LIBHAL_VOLUME_DISC_TYPE_HDDVDRW;
 		}
 	}
 
+	vol->fsusage = LIBHAL_VOLUME_USAGE_UNKNOWN;
+	if (vol_fsusage_textual != NULL) {
+		if (strcmp (vol_fsusage_textual, "filesystem") == 0) {
+			vol->fsusage = LIBHAL_VOLUME_USAGE_MOUNTABLE_FILESYSTEM;
+		} else if (strcmp (vol_fsusage_textual, "partitiontable") == 0) {
+			vol->fsusage = LIBHAL_VOLUME_USAGE_PARTITION_TABLE;
+		} else if (strcmp (vol_fsusage_textual, "raid") == 0) {
+			vol->fsusage = LIBHAL_VOLUME_USAGE_RAID_MEMBER;
+		} else if (strcmp (vol_fsusage_textual, "crypto") == 0) {
+			vol->fsusage = LIBHAL_VOLUME_USAGE_CRYPTO;
+		} else {
+			vol->fsusage = LIBHAL_VOLUME_USAGE_UNKNOWN;
+		} 
+	}
+
+	libhal_free_string (vol_fsusage_textual);
 	libhal_free_string (disc_type_textual);
 	libhal_free_property_set (properties);
 	return vol;
 error:
+	libhal_free_string (vol_fsusage_textual);
 	libhal_free_string (disc_type_textual);
 	libhal_free_property_set (properties);
 	libhal_volume_free (vol);
@@ -1112,6 +1236,8 @@ libhal_drive_from_device_file (LibHalContext *hal_ctx, const char *device_file)
 	char *found_udi;
 	DBusError error;
 
+	LIBHAL_CHECK_LIBHALCONTEXT(hal_ctx, NULL);
+
 	result = NULL;
 	found_udi = NULL;
 
@@ -1169,6 +1295,8 @@ libhal_volume_from_device_file (LibHalContext *hal_ctx, const char *device_file)
 	char *found_udi;
 	DBusError error;
 
+	LIBHAL_CHECK_LIBHALCONTEXT(hal_ctx, NULL);
+
 	result = NULL;
 	found_udi = NULL;
 
@@ -1407,6 +1535,12 @@ libhal_volume_get_disc_type (LibHalVolume *volume)
 	return volume->disc_type;
 }
 
+dbus_bool_t
+libhal_volume_should_ignore (LibHalVolume     *volume)
+{
+	return volume->ignore_volume;
+}
+
 char ** 
 libhal_drive_find_all_volumes (LibHalContext *hal_ctx, LibHalDrive *drive, int *num_volumes)
 {
@@ -1417,6 +1551,8 @@ libhal_drive_find_all_volumes (LibHalContext *hal_ctx, LibHalDrive *drive, int *
 	char **result;
 	DBusError error;
 
+	LIBHAL_CHECK_LIBHALCONTEXT(hal_ctx, NULL);
+
 	udis = NULL;
 	result = NULL;
 	*num_volumes = 0;
@@ -1442,18 +1578,59 @@ libhal_drive_find_all_volumes (LibHalContext *hal_ctx, LibHalDrive *drive, int *
 		result[*num_volumes] = strdup (udis[i]);
 		*num_volumes = (*num_volumes) + 1;
 	}
+	/* set last element (above removed UDI) to NULL for libhal_free_string_array()*/
+	result[*num_volumes] = NULL;
 
 out:
 	libhal_free_string_array (udis);
 	return result;
 }
 
+const char *
+libhal_volume_crypto_get_backing_volume_udi (LibHalVolume *volume)
+{
+	return volume->crypto_backing_volume;
+}
+
+char *
+libhal_volume_crypto_get_clear_volume_udi (LibHalContext *hal_ctx, LibHalVolume *volume)
+{
+	DBusError error;
+	char **clear_devices;
+	int num_clear_devices;
+	char *result;
+
+	result = NULL;
+
+	LIBHAL_CHECK_LIBHALCONTEXT (hal_ctx, NULL);
+
+	dbus_error_init (&error);
+	clear_devices = libhal_manager_find_device_string_match (hal_ctx,
+								 "volume.crypto_luks.clear.backing_volume",
+								 volume->udi,
+								 &num_clear_devices,
+								 &error);
+	if (clear_devices != NULL) {
+
+		if (num_clear_devices >= 1) {
+			result = strdup (clear_devices[0]);
+		}
+		libhal_free_string_array (clear_devices);
+	}
+
+	return result;
+}
+
+
 /*************************************************************************/
 
 char *
 libhal_drive_policy_default_get_mount_root (LibHalContext *hal_ctx)
 {
 	DBusError error;
+
+	LIBHAL_CHECK_LIBHALCONTEXT(hal_ctx, NULL);
+
 	dbus_error_init (&error);
 	return libhal_device_get_property_string (hal_ctx, "/org/freedesktop/Hal/devices/computer",
 						  "storage.policy.default.mount_root", &error);
@@ -1463,6 +1640,9 @@ dbus_bool_t
 libhal_drive_policy_default_use_managed_keyword (LibHalContext *hal_ctx)
 {
 	DBusError error;
+
+	LIBHAL_CHECK_LIBHALCONTEXT(hal_ctx, FALSE);
+
 	dbus_error_init (&error);
 	return libhal_device_get_property_bool (hal_ctx, "/org/freedesktop/Hal/devices/computer",
 						"storage.policy.default.use_managed_keyword", &error);
@@ -1472,6 +1652,9 @@ char *
 libhal_drive_policy_default_get_managed_keyword_primary (LibHalContext *hal_ctx)
 {
 	DBusError error;
+
+	LIBHAL_CHECK_LIBHALCONTEXT(hal_ctx, NULL);
+
 	dbus_error_init (&error);
 	return libhal_device_get_property_string (hal_ctx, "/org/freedesktop/Hal/devices/computer",
 						  "storage.policy.default.managed_keyword.primary", &error);
@@ -1481,6 +1664,9 @@ char *
 libhal_drive_policy_default_get_managed_keyword_secondary (LibHalContext *hal_ctx)
 {
 	DBusError error;
+
+	LIBHAL_CHECK_LIBHALCONTEXT(hal_ctx, NULL);
+
 	dbus_error_init (&error);
 	return libhal_device_get_property_string (hal_ctx, "/org/freedesktop/Hal/devices/computer",
 						  "storage.policy.default.managed_keyword.secondary", &error);
@@ -1517,6 +1703,11 @@ mopts_collect (LibHalContext *hal_ctx, const char *namespace, int namespace_len,
 	LibHalPropertySetIterator it;
 	DBusError error;
 
+	if(hal_ctx == 0) {
+		fprintf (stderr,"%s %d : LibHalContext *ctx is NULL\n",__FILE__, __LINE__);
+		return;
+	}
+
 	dbus_error_init (&error);
 
 	/* first collect from root computer device */
