@@ -1,11 +1,11 @@
 /***************************************************************************
- * CVSID: $Id: libhal.c,v 1.51 2005/07/12 15:02:25 david Exp $
+ * CVSID: $Id: libhal.c,v 1.57 2006/02/12 21:47:28 david Exp $
  *
  * libhal.c : HAL daemon C convenience library
  *
  * Copyright (C) 2003 David Zeuthen, <david@fubar.dk>
  *
- * Licensed under the Academic Free License version 2.0
+ * Licensed under the Academic Free License version 2.1
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -224,8 +224,7 @@ struct LibHalContext_s {
 dbus_bool_t
 libhal_ctx_set_user_data(LibHalContext *ctx, void *user_data)
 {
-	if (ctx == NULL)
-		return FALSE;
+	LIBHAL_CHECK_LIBHALCONTEXT(ctx, FALSE);
 	ctx->user_data = user_data;
 	return TRUE;
 }
@@ -239,6 +238,7 @@ libhal_ctx_set_user_data(LibHalContext *ctx, void *user_data)
 void*
 libhal_ctx_get_user_data(LibHalContext *ctx)
 {
+	LIBHAL_CHECK_LIBHALCONTEXT(ctx, NULL);
 	return ctx->user_data;
 }
 
@@ -333,13 +333,13 @@ libhal_property_fill_value_from_variant (LibHalProperty *p, DBusMessageIter *var
  *  @param  ctx                 The context for the connection to hald
  *  @param  udi                 Unique id of device
  *  @param  error               Pointer to an initialized dbus error object for
- *                              returning errors or #NULL
+ *                              returning errors or NULL
  *  @return                     An object represent all properties. Must be
  *                              freed with libhal_free_property_set
  */
 LibHalPropertySet *
 libhal_device_get_all_properties (LibHalContext *ctx, const char *udi, DBusError *error)
-{
+{	
 	DBusMessage *message;
 	DBusMessage *reply;
 	DBusMessageIter reply_iter;
@@ -348,6 +348,8 @@ libhal_device_get_all_properties (LibHalContext *ctx, const char *udi, DBusError
 	LibHalProperty *p_last;
 	DBusError _error;
 
+	LIBHAL_CHECK_LIBHALCONTEXT(ctx, NULL);
+	
 	message = dbus_message_new_method_call ("org.freedesktop.Hal", udi,
 						"org.freedesktop.Hal.Device",
 						"GetAllProperties");
@@ -501,6 +503,9 @@ libhal_property_set_get_num_elems (LibHalPropertySet *set)
 	unsigned int num_elems;
 	LibHalProperty *p;
 
+	if (set == NULL)
+		return 0;
+	
 	num_elems = 0;
 	for (p = set->properties_head; p != NULL; p = p->next)
 		num_elems++;
@@ -517,6 +522,9 @@ libhal_property_set_get_num_elems (LibHalPropertySet *set)
 void
 libhal_psi_init (LibHalPropertySetIterator * iter, LibHalPropertySet * set)
 {
+	if (set == NULL)
+		return;
+
 	iter->set = set;
 	iter->index = 0;
 	iter->cur_prop = set->properties_head;
@@ -897,7 +905,7 @@ libhal_shutdown (LibHalContext *ctx)
  *  @param  ctx                 The context for the connection to hald
  *  @param  num_devices         The number of devices will be stored here
  *  @param  error               Pointer to an initialized dbus error object for 
- *                              returning errors or #NULL
+ *                              returning errors or NULL
  *  @return                     An array of device identifiers terminated
  *                              with NULL. It is the responsibility of the
  *                              caller to free with libhal_free_string_array().
@@ -905,13 +913,15 @@ libhal_shutdown (LibHalContext *ctx)
  */
 char **
 libhal_get_all_devices (LibHalContext *ctx, int *num_devices, DBusError *error)
-{
+{	
 	DBusMessage *message;
 	DBusMessage *reply;
 	DBusMessageIter iter_array, reply_iter;
 	char **hal_device_names;
 	DBusError _error;
 
+	LIBHAL_CHECK_LIBHALCONTEXT(ctx, NULL);
+
 	*num_devices = 0;
 
 	message = dbus_message_new_method_call ("org.freedesktop.Hal",
@@ -960,7 +970,7 @@ libhal_get_all_devices (LibHalContext *ctx, int *num_devices, DBusError *error)
  *  @param  udi                 Unique Device Id
  *  @param  key                 Name of the property
  *  @param  error               Pointer to an initialized dbus error object for 
- *                              returning errors or #NULL
+ *                              returning errors or NULL
  *  @return                     One of LIBHAL_PROPERTY_TYPE_INT32,
  *                              LIBHAL_PROPERTY_TYPE_UINT64, LIBHAL_PROPERTY_TYPE_DOUBLE,
  *                              LIBHAL_PROPERTY_TYPE_BOOLEAN, LIBHAL_PROPERTY_TYPE_STRING,
@@ -976,6 +986,8 @@ libhal_device_get_property_type (LibHalContext *ctx, const char *udi, const char
 	int type;
 	DBusError _error;
 
+	LIBHAL_CHECK_LIBHALCONTEXT(ctx, LIBHAL_PROPERTY_TYPE_INVALID); // or return NULL?
+	
 	message = dbus_message_new_method_call ("org.freedesktop.Hal", udi,
 						"org.freedesktop.Hal.Device",
 						"GetPropertyType");
@@ -1017,7 +1029,7 @@ libhal_device_get_property_type (LibHalContext *ctx, const char *udi, const char
  *  @param  udi                 Unique Device Id
  *  @param  key                 Name of the property
  *  @param  error               Pointer to an initialized dbus error object for 
- *                              returning errors or #NULL
+ *                              returning errors or NULL
  *  @return                     Array of pointers to UTF8 nul-terminated
  *                              strings terminated by NULL. The caller is
  *                              responsible for freeing this string
@@ -1028,12 +1040,14 @@ libhal_device_get_property_type (LibHalContext *ctx, const char *udi, const char
  */
 char **
 libhal_device_get_property_strlist (LibHalContext *ctx, const char *udi, const char *key, DBusError *error)
-{
+{	
 	DBusMessage *message;
 	DBusMessage *reply;
 	DBusMessageIter iter, iter_array, reply_iter;
 	char **our_strings;
 	DBusError _error;
+	
+	LIBHAL_CHECK_LIBHALCONTEXT(ctx, NULL);
 
 	message = dbus_message_new_method_call ("org.freedesktop.Hal", udi,
 						"org.freedesktop.Hal.Device",
@@ -1086,7 +1100,7 @@ libhal_device_get_property_strlist (LibHalContext *ctx, const char *udi, const c
  *  @param  udi                 Unique Device Id
  *  @param  key                 Name of the property
  *  @param  error               Pointer to an initialized dbus error object for 
- *                              returning errors or #NULL
+ *                              returning errors or NULL
  *  @return                     UTF8 nul-terminated string. The caller is
  *                              responsible for freeing this string with the
  *                              function libhal_free_string(). 
@@ -1096,7 +1110,7 @@ libhal_device_get_property_strlist (LibHalContext *ctx, const char *udi, const c
 char *
 libhal_device_get_property_string (LibHalContext *ctx,
 				   const char *udi, const char *key, DBusError *error)
-{
+{	
 	DBusMessage *message;
 	DBusMessage *reply;
 	DBusMessageIter iter, reply_iter;
@@ -1104,6 +1118,8 @@ libhal_device_get_property_string (LibHalContext *ctx,
 	char *dbus_str;
 	DBusError _error;
 
+	LIBHAL_CHECK_LIBHALCONTEXT(ctx, NULL);
+
 	message = dbus_message_new_method_call ("org.freedesktop.Hal", udi,
 						"org.freedesktop.Hal.Device",
 						"GetPropertyString");
@@ -1163,7 +1179,7 @@ libhal_device_get_property_string (LibHalContext *ctx,
  *  @param  udi                 Unique Device Id
  *  @param  key                 Name of the property
  *  @param  error               Pointer to an initialized dbus error object for 
- *                              returning errors or #NULL
+ *                              returning errors or NULL
  *  @return                     Property value (32-bit signed integer)
  */
 dbus_int32_t
@@ -1176,6 +1192,8 @@ libhal_device_get_property_int (LibHalContext *ctx,
 	dbus_int32_t value;
 	DBusError _error;
 
+	LIBHAL_CHECK_LIBHALCONTEXT(ctx, -1);
+
 	message = dbus_message_new_method_call ("org.freedesktop.Hal", udi,
 						"org.freedesktop.Hal.Device",
 						"GetPropertyInteger");
@@ -1230,7 +1248,7 @@ libhal_device_get_property_int (LibHalContext *ctx,
  *  @param  udi                 Unique Device Id
  *  @param  key                 Name of the property
  *  @param  error               Pointer to an initialized dbus error object for 
- *                              returning errors or #NULL
+ *                              returning errors or NULL
  *  @return                     Property value (64-bit unsigned integer)
  */
 dbus_uint64_t
@@ -1243,6 +1261,8 @@ libhal_device_get_property_uint64 (LibHalContext *ctx,
 	dbus_uint64_t value;
 	DBusError _error;
 
+	LIBHAL_CHECK_LIBHALCONTEXT(ctx, -1);
+
 	message = dbus_message_new_method_call ("org.freedesktop.Hal", udi,
 						"org.freedesktop.Hal.Device",
 						"GetPropertyInteger");
@@ -1296,7 +1316,7 @@ libhal_device_get_property_uint64 (LibHalContext *ctx,
  *  @param  udi                 Unique Device Id
  *  @param  key                 Name of the property
  *  @param  error               Pointer to an initialized dbus error object for
- *                              returning errors or #NULL
+ *                              returning errors or NULL
  *  @return                     Property value (IEEE754 double precision float)
  */
 double
@@ -1309,6 +1329,8 @@ libhal_device_get_property_double (LibHalContext *ctx,
 	double value;
 	DBusError _error;
 
+	LIBHAL_CHECK_LIBHALCONTEXT(ctx, -1.0);
+
 	message = dbus_message_new_method_call ("org.freedesktop.Hal", udi,
 						"org.freedesktop.Hal.Device",
 						"GetPropertyDouble");
@@ -1362,7 +1384,7 @@ libhal_device_get_property_double (LibHalContext *ctx,
  *  @param  udi                 Unique Device Id
  *  @param  key                 Name of the property
  *  @param  error               Pointer to an initialized dbus error object for 
- *                              returning errors or #NULL
+ *                              returning errors or NULL
  *  @return                     Property value (boolean)
  */
 dbus_bool_t
@@ -1375,6 +1397,8 @@ libhal_device_get_property_bool (LibHalContext *ctx,
 	dbus_bool_t value;
 	DBusError _error;
 
+	LIBHAL_CHECK_LIBHALCONTEXT(ctx, FALSE);
+
 	message = dbus_message_new_method_call ("org.freedesktop.Hal", udi,
 						"org.freedesktop.Hal.Device",
 						"GetPropertyBoolean");
@@ -1441,6 +1465,8 @@ libhal_device_set_property_helper (LibHalContext *ctx,
 	DBusMessageIter iter;
 	char *method_name = NULL;
 
+	LIBHAL_CHECK_LIBHALCONTEXT(ctx, FALSE);
+	
 	/** @todo  sanity check incoming params */
 	switch (type) {
 	case DBUS_TYPE_INVALID:
@@ -1522,7 +1548,7 @@ libhal_device_set_property_helper (LibHalContext *ctx,
  *  @param  key                 Name of the property
  *  @param  value               Value of the property; a UTF8 string
  *  @param  error               Pointer to an initialized dbus error object for
- *                              returning errors or #NULL
+ *                              returning errors or NULL
  *  @return                     TRUE if the property was set, FALSE if
  *                              the device didn't exist or the property
  *                              had a different type.
@@ -1546,7 +1572,7 @@ libhal_device_set_property_string (LibHalContext *ctx,
  *  @param  key                 Name of the property
  *  @param  value               Value of the property
  *  @param  error               Pointer to an initialized dbus error object for
- *                              returning errors or #NULL
+ *                              returning errors or NULL
  *  @return                     TRUE if the property was set, FALSE if
  *                              the device didn't exist or the property
  *                              had a different type.
@@ -1567,7 +1593,7 @@ libhal_device_set_property_int (LibHalContext *ctx, const char *udi,
  *  @param  key                 Name of the property
  *  @param  value               Value of the property
  *  @param  error               Pointer to an initialized dbus error object for
- *                              returning errors or #NULL
+ *                              returning errors or NULL
  *  @return                     TRUE if the property was set, FALSE if
  *                              the device didn't exist or the property
  *                              had a different type.
@@ -1588,7 +1614,7 @@ libhal_device_set_property_uint64 (LibHalContext *ctx, const char *udi,
  *  @param  key                 Name of the property
  *  @param  value               Value of the property
  *  @param  error               Pointer to an initialized dbus error object for
- *                              returning errors or #NULL
+ *                              returning errors or NULL
  *  @return                     TRUE if the property was set, FALSE if
  *                              the device didn't exist or the property
  *                              had a different type.
@@ -1609,7 +1635,7 @@ libhal_device_set_property_double (LibHalContext *ctx, const char *udi,
  *  @param  key                 Name of the property
  *  @param  value               Value of the property
  *  @param  error               Pointer to an initialized dbus error object for
- *                              returning errors or #NULL
+ *                              returning errors or NULL
  *  @return                     TRUE if the property was set, FALSE if
  *                              the device didn't exist or the property
  *                              had a different type.
@@ -1630,7 +1656,7 @@ libhal_device_set_property_bool (LibHalContext *ctx, const char *udi,
  *  @param  udi                 Unique Device Id
  *  @param  key                 Name of the property
  *  @param  error               Pointer to an initialized dbus error object for
- *                              returning errors or #NULL
+ *                              returning errors or NULL
  *  @return                     TRUE if the property was set, FALSE if
  *                              the device didn't exist
  */
@@ -1650,7 +1676,7 @@ libhal_device_remove_property (LibHalContext *ctx,
  *  @param  key                 Name of the property
  *  @param  value               Value to append to property
  *  @param  error               Pointer to an initialized dbus error object for
- *                              returning errors or #NULL
+ *                              returning errors or NULL
  *  @return                     TRUE if the value was appended, FALSE if
  *                              the device didn't exist or the property
  *                              had a different type.
@@ -1666,6 +1692,8 @@ libhal_device_property_strlist_append (LibHalContext *ctx,
 	DBusMessage *reply;
 	DBusMessageIter iter;
 
+	LIBHAL_CHECK_LIBHALCONTEXT(ctx, FALSE);
+
 	message = dbus_message_new_method_call ("org.freedesktop.Hal", udi,
 						"org.freedesktop.Hal.Device",
 						"StringListAppend");
@@ -1700,7 +1728,7 @@ libhal_device_property_strlist_append (LibHalContext *ctx,
  *  @param  key                 Name of the property
  *  @param  value               Value to prepend to property
  *  @param  error               Pointer to an initialized dbus error object for
- *                              returning errors or #NULL
+ *                              returning errors or NULL
  *  @return                     TRUE if the value was prepended, FALSE if
  *                              the device didn't exist or the property
  *                              had a different type.
@@ -1716,6 +1744,8 @@ libhal_device_property_strlist_prepend (LibHalContext *ctx,
 	DBusMessage *reply;
 	DBusMessageIter iter;
 
+	LIBHAL_CHECK_LIBHALCONTEXT(ctx, FALSE);
+
 	message = dbus_message_new_method_call ("org.freedesktop.Hal", udi,
 						"org.freedesktop.Hal.Device",
 						"StringListPrepend");
@@ -1750,7 +1780,7 @@ libhal_device_property_strlist_prepend (LibHalContext *ctx,
  *  @param  key                 Name of the property
  *  @param  index               Index of string to remove in the strlist
  *  @param  error               Pointer to an initialized dbus error object for
- *                              returning errors or #NULL
+ *                              returning errors or NULL
  *  @return                     TRUE if the string was removed, FALSE if
  *                              the device didn't exist or the property
  *                              had a different type.
@@ -1766,6 +1796,8 @@ libhal_device_property_strlist_remove_index (LibHalContext *ctx,
 	DBusMessage *reply;
 	DBusMessageIter iter;
 
+	LIBHAL_CHECK_LIBHALCONTEXT(ctx, FALSE);
+
 	message = dbus_message_new_method_call ("org.freedesktop.Hal", udi,
 						"org.freedesktop.Hal.Device",
 						"StringListRemoveIndex");
@@ -1800,7 +1832,7 @@ libhal_device_property_strlist_remove_index (LibHalContext *ctx,
  *  @param  key                 Name of the property
  *  @param  value               The string to search for and remove
  *  @param  error               Pointer to an initialized dbus error object for
- *                              returning errors or #NULL
+ *                              returning errors or NULL
  *  @return                     TRUE if the string was removed, FALSE if
  *                              the device didn't exist or the property
  *                              had a different type.
@@ -1815,6 +1847,8 @@ libhal_device_property_strlist_remove (LibHalContext *ctx,
 	DBusMessage *reply;
 	DBusMessageIter iter;
 
+	LIBHAL_CHECK_LIBHALCONTEXT(ctx, FALSE);
+
 	message = dbus_message_new_method_call ("org.freedesktop.Hal", udi,
 						"org.freedesktop.Hal.Device",
 						"StringListRemove");
@@ -1853,7 +1887,7 @@ libhal_device_property_strlist_remove (LibHalContext *ctx,
  *                              device cannot be locked on failure, or
  *                              NULL
  *  @param  error               Pointer to an initialized dbus error object for
- *                              returning errors or #NULL
+ *                              returning errors or NULL
  *  @return                     TRUE if the lock was obtained, FALSE
  *                              otherwise
  */
@@ -1867,6 +1901,8 @@ libhal_device_lock (LibHalContext *ctx,
 	DBusMessageIter iter;
 	DBusMessage *reply;
 
+	LIBHAL_CHECK_LIBHALCONTEXT(ctx, FALSE);
+
 	if (reason_why_locked != NULL)
 		*reason_why_locked = NULL;
 
@@ -1919,7 +1955,7 @@ libhal_device_lock (LibHalContext *ctx,
  *  @param  ctx                 The context for the connection to hald
  *  @param  udi                 Unique Device Id
  *  @param  error               Pointer to an initialized dbus error object for
- *                              returning errors or #NULL
+ *                              returning errors or NULL
  *  @return                     TRUE if the device was successfully unlocked,
  *                              FALSE otherwise
  */
@@ -1930,6 +1966,8 @@ libhal_device_unlock (LibHalContext *ctx,
 	DBusMessage *message;
 	DBusMessage *reply;
 
+	LIBHAL_CHECK_LIBHALCONTEXT(ctx, FALSE);
+
 	message = dbus_message_new_method_call ("org.freedesktop.Hal",
 						udi,
 						"org.freedesktop.Hal.Device",
@@ -1971,7 +2009,7 @@ libhal_device_unlock (LibHalContext *ctx,
  *
  *  @param  ctx                 The context for the connection to hald
  *  @param  error               Pointer to an initialized dbus error object for
- *                              returning errors or #NULL
+ *                              returning errors or NULL
  *  @return                     Temporary device unique id or NULL if there
  *                              was a problem. This string must be freed
  *                              by the caller.
@@ -1985,6 +2023,8 @@ libhal_new_device (LibHalContext *ctx, DBusError *error)
 	char *value;
 	char *dbus_str;
 
+	LIBHAL_CHECK_LIBHALCONTEXT(ctx, NULL);
+
 	message = dbus_message_new_method_call ("org.freedesktop.Hal",
 						"/org/freedesktop/Hal/Manager",
 						"org.freedesktop.Hal.Manager",
@@ -2052,7 +2092,7 @@ libhal_new_device (LibHalContext *ctx, DBusError *error)
  *                              libhal_new_device()
  *  @param  udi                 The new unique device id.
  *  @param  error               Pointer to an initialized dbus error object for
- *                              returning errors or #NULL
+ *                              returning errors or NULL
  *  @return                     FALSE if the given unique device id is already
  *                              in use.
  */
@@ -2064,6 +2104,8 @@ libhal_device_commit_to_gdl (LibHalContext *ctx,
 	DBusMessage *reply;
 	DBusMessageIter iter;
 
+	LIBHAL_CHECK_LIBHALCONTEXT(ctx, FALSE);
+
 	message = dbus_message_new_method_call ("org.freedesktop.Hal",
 						"/org/freedesktop/Hal/Manager",
 						"org.freedesktop.Hal.Manager",
@@ -2107,7 +2149,7 @@ libhal_device_commit_to_gdl (LibHalContext *ctx,
  *  @param  ctx                 The context for the connection to hald
  *  @param  udi                 The new unique device id.
  *  @param  error               Pointer to an initialized dbus error object for
- *                              returning errors or #NULL
+ *                              returning errors or NULL
  *  @return                     TRUE if the device was removed, FALSE otherwise
  */
 dbus_bool_t
@@ -2117,6 +2159,8 @@ libhal_remove_device (LibHalContext *ctx, const char *udi, DBusError *error)
 	DBusMessage *reply;
 	DBusMessageIter iter;
 
+	LIBHAL_CHECK_LIBHALCONTEXT(ctx, FALSE);
+
 	message = dbus_message_new_method_call ("org.freedesktop.Hal",
 						"/org/freedesktop/Hal/Manager",
 						"org.freedesktop.Hal.Manager",
@@ -2154,7 +2198,7 @@ libhal_remove_device (LibHalContext *ctx, const char *udi, DBusError *error)
  *  @param  ctx                 The context for the connection to hald
  *  @param  udi                 Unique device id.
  *  @param  error               Pointer to an initialized dbus error object for 
- *                              returning errors or #NULL
+ *                              returning errors or NULL
  *  @return                     TRUE if the device exists
  */
 dbus_bool_t
@@ -2166,6 +2210,8 @@ libhal_device_exists (LibHalContext *ctx, const char *udi, DBusError *error)
 	dbus_bool_t value;
 	DBusError _error;
 
+	LIBHAL_CHECK_LIBHALCONTEXT(ctx, FALSE);
+
 	message = dbus_message_new_method_call ("org.freedesktop.Hal",
 						"/org/freedesktop/Hal/Manager",
 						"org.freedesktop.Hal.Manager",
@@ -2220,7 +2266,7 @@ libhal_device_exists (LibHalContext *ctx, const char *udi, DBusError *error)
  *  @param  udi                 Unique device id.
  *  @param  key                 Name of the property
  *  @param  error               Pointer to an initialized dbus error object for 
- *                              returning errors or #NULL
+ *                              returning errors or NULL
  *  @return                     TRUE if the device exists, FALSE otherwise
  */
 dbus_bool_t
@@ -2233,6 +2279,8 @@ libhal_device_property_exists (LibHalContext *ctx,
 	dbus_bool_t value;
 	DBusError _error;
 
+	LIBHAL_CHECK_LIBHALCONTEXT(ctx, FALSE);
+
 	message = dbus_message_new_method_call ("org.freedesktop.Hal", udi,
 						"org.freedesktop.Hal.Device",
 						"PropertyExists");
@@ -2285,7 +2333,7 @@ libhal_device_property_exists (LibHalContext *ctx,
  *  @param  target_udi          Unique device id of target device to merge to
  *  @param  source_udi          Unique device id of device to merge from
  *  @param  error               Pointer to an initialized dbus error object for 
- *                              returning errors or #NULL
+ *                              returning errors or NULL
  *  @return                     TRUE if the properties were merged, FALSE otherwise
  */
 dbus_bool_t
@@ -2296,6 +2344,8 @@ libhal_merge_properties (LibHalContext *ctx,
 	DBusMessage *reply;
 	DBusMessageIter iter;
 
+	LIBHAL_CHECK_LIBHALCONTEXT(ctx, FALSE);
+
 	message = dbus_message_new_method_call ("org.freedesktop.Hal",
 						"/org/freedesktop/Hal/Manager",
 						"org.freedesktop.Hal.Manager",
@@ -2344,7 +2394,7 @@ libhal_merge_properties (LibHalContext *ctx,
  *  @param  udi2                Unique Device Id for device 2
  *  @param  property_namespace  Namespace for set of devices, e.g. "usb"
  *  @param  error               Pointer to an initialized dbus error object for 
- *                              returning errors or #NULL
+ *                              returning errors or NULL
  *  @return                     TRUE if all properties starting
  *                              with the given namespace parameter
  *                              from one device is in the other and 
@@ -2361,6 +2411,8 @@ libhal_device_matches (LibHalContext *ctx,
 	dbus_bool_t value;
 	DBusError _error;
 
+	LIBHAL_CHECK_LIBHALCONTEXT(ctx, FALSE);
+
 	message = dbus_message_new_method_call ("org.freedesktop.Hal",
 						"/org/freedesktop/Hal/Manager",
 						"org.freedesktop.Hal.Manager",
@@ -2415,7 +2467,7 @@ libhal_device_matches (LibHalContext *ctx,
  *  @param  ctx                 The context for the connection to hald
  *  @param  udi                 Unique Device Id
  *  @param  error               Pointer to an initialized dbus error object for
- *                              returning errors or #NULL
+ *                              returning errors or NULL
  *  @return                     TRUE if device's information could be obtained,
  *                              FALSE otherwise
  */
@@ -2427,6 +2479,8 @@ libhal_device_print (LibHalContext *ctx, const char *udi, DBusError *error)
 	LibHalPropertySet *pset;
 	LibHalPropertySetIterator i;
 
+	LIBHAL_CHECK_LIBHALCONTEXT(ctx, FALSE);
+
 	printf ("device_id = %s\n", udi);
 
 	if ((pset = libhal_device_get_all_properties (ctx, udi, error)) == NULL)
@@ -2496,7 +2550,7 @@ libhal_device_print (LibHalContext *ctx, const char *udi, DBusError *error)
  *  @param  value               Value to match
  *  @param  num_devices         Pointer to store number of devices
  *  @param  error               Pointer to an initialized dbus error object for 
- *                              returning errors or #NULL
+ *                              returning errors or NULL
  *  @return                     UDI of devices; free with 
  *                              libhal_free_string_array()
  */
@@ -2511,6 +2565,8 @@ libhal_manager_find_device_string_match (LibHalContext *ctx,
 	char **hal_device_names;
 	DBusError _error;
 
+	LIBHAL_CHECK_LIBHALCONTEXT(ctx, NULL);
+
 	message = dbus_message_new_method_call ("org.freedesktop.Hal",
 						"/org/freedesktop/Hal/Manager",
 						"org.freedesktop.Hal.Manager",
@@ -2565,7 +2621,7 @@ libhal_manager_find_device_string_match (LibHalContext *ctx,
  *  @param  udi                 Unique Device Id
  *  @param  capability          Capability name
  *  @param  error               Pointer to an initialized dbus error object for 
- *                              returning errors or #NULL
+ *                              returning errors or NULL
  *  @return                     TRUE if the capability was added, FALSE if
  *                              the device didn't exist
  */
@@ -2577,6 +2633,8 @@ libhal_device_add_capability (LibHalContext *ctx,
 	DBusMessage *reply;
 	DBusMessageIter iter;
 
+	LIBHAL_CHECK_LIBHALCONTEXT(ctx, FALSE);
+
 	message = dbus_message_new_method_call ("org.freedesktop.Hal", udi,
 						"org.freedesktop.Hal.Device",
 						"AddCapability");
@@ -2616,7 +2674,7 @@ libhal_device_add_capability (LibHalContext *ctx,
  *  @param  udi                 Unique Device Id
  *  @param  capability          Capability name
  *  @param  error               Pointer to an initialized dbus error object for 
- *                              returning errors or #NULL
+ *                              returning errors or NULL
  *  @return                     TRUE if the device has the capability,
  *                              otherwise FALSE
  */
@@ -2627,6 +2685,8 @@ libhal_device_query_capability (LibHalContext *ctx, const char *udi, const char
 	unsigned int i;
 	dbus_bool_t ret;
 
+	LIBHAL_CHECK_LIBHALCONTEXT(ctx, FALSE);
+
 	ret = FALSE;
 
 	caps = libhal_device_get_property_strlist (ctx, udi, "info.capabilities", error);
@@ -2649,7 +2709,7 @@ libhal_device_query_capability (LibHalContext *ctx, const char *udi, const char
  *  @param  capability          Capability name
  *  @param  num_devices         Pointer to store number of devices
  *  @param  error               Pointer to an initialized dbus error object for 
- *                              returning errors or #NULL
+ *                              returning errors or NULL
  *  @return                     UDI of devices; free with 
  *                              libhal_free_string_array()
  */
@@ -2663,6 +2723,8 @@ libhal_find_device_by_capability (LibHalContext *ctx,
 	char **hal_device_names;
 	DBusError _error;
 
+	LIBHAL_CHECK_LIBHALCONTEXT(ctx, NULL);
+
 	message = dbus_message_new_method_call ("org.freedesktop.Hal",
 						"/org/freedesktop/Hal/Manager",
 						"org.freedesktop.Hal.Manager",
@@ -2714,12 +2776,14 @@ libhal_find_device_by_capability (LibHalContext *ctx,
  *
  *  @param  ctx                 The context for the connection to hald
  *  @param  error               Pointer to an initialized dbus error object for 
- *                              returning errors or #NULL
+ *                              returning errors or NULL
  *  @return                     TRUE only if the operation succeeded
  */
 dbus_bool_t
 libhal_device_property_watch_all (LibHalContext *ctx, DBusError *error)
 {
+	LIBHAL_CHECK_LIBHALCONTEXT(ctx, FALSE);
+
 	dbus_bus_add_match (ctx->connection,
 			    "type='signal',"
 			    "interface='org.freedesktop.Hal.Device',"
@@ -2740,13 +2804,15 @@ libhal_device_property_watch_all (LibHalContext *ctx, DBusError *error)
  *  @param  ctx                 The context for the connection to hald
  *  @param  udi                 Unique Device Id
  *  @param  error               Pointer to an initialized dbus error object for 
- *                              returning errors or #NULL
+ *                              returning errors or NULL
  *  @return                     TRUE only if the operation succeeded
  */
 dbus_bool_t
 libhal_device_add_property_watch (LibHalContext *ctx, const char *udi, DBusError *error)
-{
+{	
 	char buf[512];
+	
+	LIBHAL_CHECK_LIBHALCONTEXT(ctx, FALSE);
 
 	snprintf (buf, 512,
 		  "type='signal',"
@@ -2766,13 +2832,15 @@ libhal_device_add_property_watch (LibHalContext *ctx, const char *udi, DBusError
  *  @param  ctx                 The context for the connection to hald
  *  @param  udi                 Unique Device Id
  *  @param  error               Pointer to an initialized dbus error object for 
- *                              returning errors or #NULL
+ *                              returning errors or NULL
  *  @return                     TRUE only if the operation succeeded
  */
 dbus_bool_t
 libhal_device_remove_property_watch (LibHalContext *ctx, const char *udi, DBusError *error)
-{
+{	
 	char buf[512];
+	
+	LIBHAL_CHECK_LIBHALCONTEXT(ctx, FALSE);
 
 	snprintf (buf, 512,
 		  "type='signal',"
@@ -2805,7 +2873,9 @@ libhal_ctx_new (void)
 
 	ctx = calloc (1, sizeof (LibHalContext));
 	if (ctx == NULL) {
-		fprintf (stderr, "%s %d : Failed to allocate %d bytes\n", __FILE__, __LINE__, sizeof (LibHalContext));
+		fprintf (stderr, 
+			 "%s %d : Failed to allocate %d bytes\n",
+			 __FILE__, __LINE__, sizeof (LibHalContext));
 		return NULL;
 	}
 
@@ -2829,8 +2899,7 @@ libhal_ctx_new (void)
 dbus_bool_t
 libhal_ctx_set_cache (LibHalContext *ctx, dbus_bool_t use_cache)
 {
-	if (ctx == NULL)
-		return FALSE;
+	LIBHAL_CHECK_LIBHALCONTEXT(ctx, FALSE);
 
 	ctx->cache_enabled = use_cache;
 	return TRUE;
@@ -2846,8 +2915,7 @@ libhal_ctx_set_cache (LibHalContext *ctx, dbus_bool_t use_cache)
 dbus_bool_t
 libhal_ctx_set_dbus_connection (LibHalContext *ctx, DBusConnection *conn)
 {
-	if (ctx == NULL)
-		return FALSE;
+	LIBHAL_CHECK_LIBHALCONTEXT(ctx, FALSE);
 
 	if (conn == NULL)
 		return FALSE;
@@ -2856,12 +2924,26 @@ libhal_ctx_set_dbus_connection (LibHalContext *ctx, DBusConnection *conn)
 	return TRUE;
 }
 
+/** Get DBus connection to use to talk to hald.
+ *
+ *  @param  ctx                 Context to set connection for
+ *  @return conn                DBus connection to use or NULL
+ */
+DBusConnection *
+libhal_ctx_get_dbus_connection (LibHalContext *ctx)
+{
+	LIBHAL_CHECK_LIBHALCONTEXT(ctx, FALSE);
+
+	return ctx->connection;
+}
+
+
 /** Initialize the connection to hald
  *
  *  @param  ctx                 Context for connection to hald (connection
  *                              should be set with libhal_ctx_set_dbus_connection)
  *  @param  error               Pointer to an initialized dbus error object for 
- *                              returning errors or #NULL
+ *                              returning errors or NULL
  *  @return                     TRUE if initialization succeeds, FALSE otherwise
  */
 dbus_bool_t 
@@ -2870,8 +2952,7 @@ libhal_ctx_init (LibHalContext *ctx, DBusError *error)
 	DBusError _error;
 	dbus_bool_t hald_exists;
 
-	if (ctx == NULL)
-		return FALSE;
+	LIBHAL_CHECK_LIBHALCONTEXT(ctx, FALSE);
 
 	if (ctx->connection == NULL)
 		return FALSE;
@@ -2910,7 +2991,7 @@ libhal_ctx_init (LibHalContext *ctx, DBusError *error)
 /** Create an already initialized connection to hald
  *
  *  @param  error               Pointer to an initialized dbus error object for 
- *                              returning errors or #NULL
+ *                              returning errors or NULL
  *  @return                     A pointer to an already initialized LibHalContext
  */
 LibHalContext *
@@ -2950,15 +3031,17 @@ out:
  *
  *  @param  ctx                 Context for connection to hald
  *  @param  error               Pointer to an initialized dbus error object for 
- *                              returning errors or #NULL
+ *                              returning errors or NULL
  *  @return                     TRUE if connection successfully shut down,
  *                              FALSE otherwise
  */
 dbus_bool_t    
 libhal_ctx_shutdown (LibHalContext *ctx, DBusError *error)
-{
+{	
 	DBusError myerror;
 
+	LIBHAL_CHECK_LIBHALCONTEXT(ctx, FALSE);
+
 	if (ctx->is_direct) {
 		/* for some reason dbus_connection_set_exit_on_disconnect doesn't work yet so don't unref */
 		/*dbus_connection_unref (ctx->connection);*/
@@ -3007,8 +3090,8 @@ libhal_ctx_free (LibHalContext *ctx)
 dbus_bool_t
 libhal_ctx_set_device_added (LibHalContext *ctx, LibHalDeviceAdded callback)
 {
-	if (ctx == NULL)
-		return FALSE;
+	LIBHAL_CHECK_LIBHALCONTEXT(ctx, FALSE);
+	
 	ctx->device_added = callback;
 	return TRUE;
 }
@@ -3023,8 +3106,8 @@ libhal_ctx_set_device_added (LibHalContext *ctx, LibHalDeviceAdded callback)
 dbus_bool_t
 libhal_ctx_set_device_removed (LibHalContext *ctx, LibHalDeviceRemoved callback)
 {
-	if (ctx == NULL)
-		return FALSE;
+	LIBHAL_CHECK_LIBHALCONTEXT(ctx, FALSE);
+	
 	ctx->device_removed = callback;
 	return TRUE;
 }
@@ -3040,8 +3123,8 @@ libhal_ctx_set_device_removed (LibHalContext *ctx, LibHalDeviceRemoved callback)
 dbus_bool_t
 libhal_ctx_set_device_new_capability (LibHalContext *ctx, LibHalDeviceNewCapability callback)
 {
-	if (ctx == NULL)
-		return FALSE;
+	LIBHAL_CHECK_LIBHALCONTEXT(ctx, FALSE);
+	
 	ctx->device_new_capability = callback;
 	return TRUE;
 }
@@ -3057,8 +3140,8 @@ libhal_ctx_set_device_new_capability (LibHalContext *ctx, LibHalDeviceNewCapabil
 dbus_bool_t
 libhal_ctx_set_device_lost_capability (LibHalContext *ctx, LibHalDeviceLostCapability callback)
 {
-	if (ctx == NULL)
-		return FALSE;
+	LIBHAL_CHECK_LIBHALCONTEXT(ctx, FALSE);
+	
 	ctx->device_lost_capability = callback;
 	return TRUE;
 }
@@ -3074,8 +3157,8 @@ libhal_ctx_set_device_lost_capability (LibHalContext *ctx, LibHalDeviceLostCapab
 dbus_bool_t
 libhal_ctx_set_device_property_modified (LibHalContext *ctx, LibHalDevicePropertyModified callback)
 {
-	if (ctx == NULL)
-		return FALSE;
+	LIBHAL_CHECK_LIBHALCONTEXT(ctx, FALSE);
+	
 	ctx->device_property_modified = callback;
 	return TRUE;
 }
@@ -3090,8 +3173,8 @@ libhal_ctx_set_device_property_modified (LibHalContext *ctx, LibHalDevicePropert
 dbus_bool_t
 libhal_ctx_set_device_condition (LibHalContext *ctx, LibHalDeviceCondition callback)
 {
-	if (ctx == NULL)
-		return FALSE;
+	LIBHAL_CHECK_LIBHALCONTEXT(ctx, FALSE);
+	
 	ctx->device_condition = callback;
 	return TRUE;
 }
@@ -3118,6 +3201,8 @@ unsigned int libhal_string_array_length (char **str_array)
 dbus_bool_t 
 libhal_device_rescan (LibHalContext *ctx, const char *udi, DBusError *error)
 {
+	LIBHAL_CHECK_LIBHALCONTEXT(ctx, FALSE);
+	
 	DBusMessage *message;
 	DBusMessageIter reply_iter;
 	DBusMessage *reply;
@@ -3165,6 +3250,8 @@ libhal_device_rescan (LibHalContext *ctx, const char *udi, DBusError *error)
 dbus_bool_t
 libhal_device_reprobe (LibHalContext *ctx, const char *udi, DBusError *error)
 {
+	LIBHAL_CHECK_LIBHALCONTEXT(ctx, FALSE);
+	
 	DBusMessage *message;
 	DBusMessageIter reply_iter;
 	DBusMessage *reply;
@@ -3217,7 +3304,7 @@ libhal_device_reprobe (LibHalContext *ctx, const char *udi, DBusError *error)
  *  @param  condition_name      User-readable name of condition
  *  @param  condition_details   User-readable details of condition
  *  @param  error               Pointer to an initialized dbus error object for 
- *                              returning errors or #NULL
+ *                              returning errors or NULL
  *  @return                     TRUE if condition successfully emitted,
  *                              FALSE otherwise
  */
@@ -3227,6 +3314,8 @@ dbus_bool_t libhal_device_emit_condition (LibHalContext *ctx,
 					  const char *condition_details,
 					  DBusError *error)
 {
+	LIBHAL_CHECK_LIBHALCONTEXT(ctx, FALSE);
+	
 	DBusMessage *message;
 	DBusMessageIter iter;
 	DBusMessageIter reply_iter;
