@@ -1,11 +1,11 @@
 /***************************************************************************
- * CVSID: $Id: libhal.h,v 1.28 2005/07/08 23:02:29 david Exp $
+ * CVSID: $Id: libhal.h,v 1.32 2006/02/12 21:47:28 david Exp $
  *
  * libhal.h : HAL daemon C convenience library headers
  *
  * Copyright (C) 2003 David Zeuthen, <david@fubar.dk>
  *
- * Licensed under the Academic Free License version 2.0
+ * Licensed under the Academic Free License version 2.1
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -19,7 +19,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  *
  **************************************************************************/
 
@@ -35,6 +35,17 @@ extern "C" {
 #endif
 #endif
 
+/** Checks if LibHalContext *ctx == NULL */
+#define LIBHAL_CHECK_LIBHALCONTEXT(_ctx_, _ret_)					\
+	do {									\
+		if (_ctx_ == NULL) {						\
+			fprintf (stderr,					\
+				 "%s %d : LibHalContext *ctx is NULL\n", 	\
+				 __FILE__, __LINE__);				\
+			return _ret_;						\
+		}								\
+	} while(0)
+
 /**
  * @addtogroup LibHal
  *
@@ -152,6 +163,9 @@ dbus_bool_t    libhal_ctx_set_cache                    (LibHalContext *ctx, dbus
 /* Set DBus connection to use to talk to hald. */
 dbus_bool_t    libhal_ctx_set_dbus_connection          (LibHalContext *ctx, DBusConnection *conn);
 
+/* Get DBus connection to use to talk to hald. */
+DBusConnection *libhal_ctx_get_dbus_connection          (LibHalContext *ctx);
+
 /* Set user data for the context */
 dbus_bool_t    libhal_ctx_set_user_data                (LibHalContext *ctx, void *user_data);
 
