@@ -3,19 +3,9 @@
  *
  * Copyright (C) 2005 Kay Sievers <kay.sievers@vrfy.org>
  *
- *	This library is free software; you can redistribute it and/or
- *	modify it under the terms of the GNU Lesser General Public
- *	License as published by the Free Software Foundation; either
- *	version 2.1 of the License, or (at your option) any later version.
- *
- *	This library is distributed in the hope that it will be useful,
- *	but WITHOUT ANY WARRANTY; without even the implied warranty of
- *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
- *	Lesser General Public License for more details.
- *
- *	You should have received a copy of the GNU Lesser General Public
- *	License along with this library; if not, write to the Free Software
- *	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
+ *	This program is free software; you can redistribute it and/or modify it
+ *	under the terms of the GNU General Public License as published by the
+ *	Free Software Foundation version 2 of the License.
  */
 
 #ifndef _GNU_SOURCE
@@ -36,21 +26,20 @@
 #include "volume_id.h"
 #include "logging.h"
 #include "util.h"
-#include "nvidia_raid.h"
 
 struct nvidia_meta {
-	__u8	vendor[8];
-	__u32	size;
-	__u32	chksum;
-	__u16	version;
-} __attribute__((packed));
+	uint8_t		vendor[8];
+	uint32_t	size;
+	uint32_t	chksum;
+	uint16_t	version;
+} PACKED;
 
 #define NVIDIA_SIGNATURE		"NVIDIA"
 
-int volume_id_probe_nvidia_raid(struct volume_id *id, __u64 off, __u64 size)
+int volume_id_probe_nvidia_raid(struct volume_id *id, uint64_t off, uint64_t size)
 {
-	const __u8 *buf;
-	__u64 meta_off;
+	const uint8_t *buf;
+	uint64_t meta_off;
 	struct nvidia_meta *nv;
 
 	dbg("probing at offset 0x%llx, size 0x%llx",
